@@ -20,12 +20,6 @@ import re
 
 class ConsoleEvent(EventBase):
 
-    def __deepcopy__(self, a):
-        """ deepcopy an console event. """
-        e = ConsoleEvent()
-        e.copyin(self)
-        return e
-
     def parse(self, bot, input, console, *args, **kwargs):
         """ overload this. """
         if not input: raise NoInput()
@@ -36,7 +30,9 @@ class ConsoleEvent(EventBase):
         self.userhost = self.auth
         self.origin = self.userhost
         self.txt = input
-        self.usercmnd = input.split()[0]
         self.channel = self.userhost
         self.cbtype = self.cmnd = unicode("CONSOLE")
         self.showall = True
+        self.prepare()
+        return self
+        
\ No newline at end of file
