@@ -54,7 +54,7 @@ class Irc(BotBase):
 
     def __init__(self, cfg=None, users=None, plugs=None, *args, **kwargs):
         BotBase.__init__(self, cfg, users, plugs, *args, **kwargs)
-        BotBase.setstate(self)
+        #BotBase.setstate(self)
         self.type = 'irc'
         self.fsock = None
         self.oldsock = None
@@ -80,14 +80,14 @@ class Irc(BotBase):
     def _raw(self, txt):
         """ send raw text to the server. """
         if not txt or self.stopped or not self.sock:
-            logging.info("%s - bot is stopped .. not sending." % self.cfg.name)
+            logging.debug("%s - bot is stopped .. not sending." % self.cfg.name)
             return 0
         try:
             self.lastoutput = time.time()
             itxt = toenc(txt, self.encoding)
-            if not self.sock: logging.warn("%s - socket disappeared - not sending." % self.cfg.name) ; return
-            #if not txt.startswith("PONG"): logging.warn("> %s" % itxt)             
-            logging.warn("> %s" % itxt)             
+            if not self.sock: logging.debug("%s - socket disappeared - not sending." % self.cfg.name) ; return
+            if not txt.startswith("PONG"): logging.warn("> %s (%s)" % (itxt, self.cfg.name))
+            else: logging.info("> %s (%s)" % (itxt, self.cfg.name))             
             if self.cfg.has_key('ssl') and self.cfg['ssl']: self.sock.write(itxt + '\n')
             else: self.sock.send(itxt[:500] + '\n')
         except Exception, ex: logging.error("%s - can't send: %s" % (self.cfg.name, str(ex)))
@@ -105,12 +105,12 @@ class Irc(BotBase):
         assert self.cfg.server
         assert self.cfg.port
         server = self.bind()
-        logging.warn('%s - connecting to %s - %s - %s' % (self.cfg.name, server, self.cfg.server, self.cfg.port))
+        logging.warn('connecting to %s - %s - %s (%s)' % (server, self.cfg.server, self.cfg.port, self.cfg.name))
         self.oldsock.settimeout(30)
         self.oldsock.connect((server, int(str(self.cfg.port))))	
         self.blocking = 1
         self.oldsock.setblocking(self.blocking)
-        logging.warn('%s - connection ok' % self.cfg.name)
+        logging.warn('connected! (%s)' % self.cfg.name)
         self.connected = True
         self.fsock = self.oldsock.makefile("r")
         self.fsock._sock.setblocking(self.blocking)
@@ -123,7 +123,7 @@ class Irc(BotBase):
             self.oldsock.settimeout(socktimeout)
             self.fsock._sock.settimeout(socktimeout)
         if self.cfg.has_key('ssl') and self.cfg['ssl']:
-            logging.info('%s - ssl enabled' % self.cfg.name)
+            logging.warn('ssl enabled (%s)' % self.cfg.name)
             self.sock = socket.ssl(self.oldsock) 
         else: self.sock = self.oldsock
         try:
@@ -137,10 +137,11 @@ class Irc(BotBase):
         server = self.cfg.server
         elite = self.cfg['bindhost'] or getmainconfig()['bindhost']
         if elite:
+            logging.warn("trying to bind to %s" % elite)
             try:
                 self.oldsock.bind((elite, 0))
             except socket.gaierror:
-                logging.warn("%s - can't bind to %s" % (self.cfg.name, elite))
+                logging.debug("%s - can't bind to %s" % (self.cfg.name, elite))
                 if not server:
                     try: socket.inet_pton(socket.AF_INET6, self.cfg.server)
                     except socket.error: pass
@@ -166,22 +167,18 @@ class Irc(BotBase):
         self.stopped = False
         doreconnect = True
         timeout = 1
-        logging.warn('%s - starting readloop' % self.cfg.name)
+        logging.debug('%s - starting readloop' % self.cfg.name)
         prevtxt = ""
         while not self.stopped and not self.stopreadloop and self.sock and self.fsock:
             try:
                 time.sleep(0.01)
                 if self.cfg.has_key('ssl') and self.cfg['ssl']: intxt = inputmorphs.do(self.sock.read()).split('\n')
-                #else: intxt = inputmorphs.do(self.fsock.readline()).split('\n')
-		else:
-                    try: [i, o, e] = select.select([self.sock], [], [])
-                    except socket.error: doreconnect = False ; break
-                    if i: intxt = inputmorphs.do(self.sock.recv(600)).split('\n')
-                    if self.stopreadloop or self.stopped:
-                        doreconnect = 0
-                        break
-                    if not i: time.sleep(0.1) ; continue
-                if not intxt:
+                else: intxt = inputmorphs.do(self.fsock.readline()).split('\n')
+                if self.stopreadloop or self.stopped:
+                    doreconnect = 0
+                    break
+                if intxt == ["",]:
+                    logging.error("remote disconnected")
                     doreconnect = 1
                     break
                 if prevtxt:
@@ -196,31 +193,34 @@ class Irc(BotBase):
                         r = strippedtxt(r.rstrip(), ["\001", "\002", "\003"]) 
                         rr = unicode(fromenc(r.rstrip(), self.encoding))
                     except UnicodeDecodeError:
-                        logging.warn("%s - decode error - ignoring" % self.cfg.name)
+                        logging.warn("decode error - ignoring (%s)" % self.cfg.name)
                         continue
                     if not rr: continue
                     res = rr
-                    #if not res.startswith("PING"): logging.warn(u"< %s" % res)
-                    #else: logging.info(u"< %s" % res)
-                    logging.warn(u"< %s" % res)
                     try:
                         ievent = IrcEvent().parse(self, res)
                     except Exception, ex:
                         handle_exception()
                         continue
+                    try:
+                        if int(ievent.cmnd) > 400: logging.error(u"< %s (%s)" % (res, self.cfg.name))
+                        elif int(ievent.cmnd) >= 300: logging.info(u"< %s (%s)" % (res, self.cfg.name))
+                    except ValueError:
+                        if not res.startswith("PING") and not res.startswith("NOTICE"): logging.warn(u"< %s (%s)" % (res, self.cfg.name))
+                        else: logging.info(u"< %s (%s)" % (res, self.cfg.name))
                     if ievent: self.handle_ievent(ievent)
                     timeout = 1
             except UnicodeError:
                 handle_exception()
                 continue
             except socket.timeout, ex:
-                logging.warn("%s - socket timeout" % self.cfg.name)
+                logging.warn("socket timeout (%s)" % self.cfg.name)
                 self.error = str(ex)
                 if self.stopped or self.stopreadloop: break
                 timeout += 1
                 if timeout > 2:
                     doreconnect = 1
-                    logging.warn('%s - no pong received' % self.cfg.name)
+                    logging.warn('no pong received (%s)' % self.cfg.name)
                     break
                 pingsend = self.ping()
                 if not pingsend:
@@ -237,9 +237,9 @@ class Irc(BotBase):
                 timeout += 1
                 if timeout > 2:
                     doreconnect = 1
-                    logging.warn('%s - no pong received' % self.cfg.name)
+                    logging.warn('no pong received (%s)' % self.cfg.name)
                     break
-                logging.warn("%s - socket timeout" % self.cfg.name)
+                logging.warn("socket timeout (%s)" % self.cfg.name)
                 pingsend = self.ping()
                 if not pingsend:
                     doreconnect = 1
@@ -248,23 +248,24 @@ class Irc(BotBase):
             except IOError, ex:
                 self.error = str(ex)
                 if self.blocking and 'temporarily' in str(ex):
-                    logging.warn("IOError: %s" % self.error)
+                    logging.warn("iorror: %s (%s)" % (self.error, self.cfg.name))
                     time.sleep(1)
                     continue
                 if not self.stopped:
-                    logging.error('%s - connecting error: %s' % (self.cfg.name, str(ex)))
+                    logging.error('connecting error: %s (%s)' % (str(ex), self.cfg.name))
                     handle_exception()
                     doreconnect = 1
                 break
             except socket.error, ex:
                 self.error = str(ex)
                 if self.blocking and 'temporarily' in str(ex):
-                    logging.warn("IOError: %s" % self.error)
+                    logging.warn("ioerror: %s (%s)" % (self.error, self.cfg.name))
                     time.sleep(0.5)
                     continue
                 if not self.stopped:
-                    logging.error('%s - connecting error: %s' % (self.cfg.name, str(ex)))
+                    logging.error('connecting error: %s (%s)' % (str(ex), self.cfg.name))
                     doreconnect = 1
+                break
             except Exception, ex:
                 self.error = str(ex)
                 if self.stopped or self.stopreadloop:
@@ -272,7 +273,7 @@ class Irc(BotBase):
                 logging.error("%s - error in readloop: %s" % (self.cfg.name, str(ex)))
                 doreconnect = 1
                 break
-        logging.warn('%s - readloop stopped - %s' % (self.cfg.name, self.error))
+        logging.debug('%s - readloop stopped - %s' % (self.cfg.name, self.error))
         self.connectok.clear()
         self.connected = False
         if doreconnect and not self.stopped:
@@ -285,7 +286,7 @@ class Irc(BotBase):
         if self.cfg.password:
             logging.debug('%s - sending password' % self.cfg.name)
             self._raw("PASS %s" % self.cfg.password)
-        logging.warn('%s - registering with %s using nick %s' % (self.cfg.name, self.cfg.server, self.cfg.nick))
+        logging.warn('registering with %s using nick %s (%s)' % (self.cfg.server, self.cfg.nick, self.cfg.name))
         logging.warn('%s - this may take a while' % self.cfg.name)
         username = self.cfg.username or "jsb"
         realname = self.cfg.realname or "jsonbot"
@@ -296,7 +297,14 @@ class Irc(BotBase):
 
     def _onconnect(self):
         """ overload this to run after connect. """
-        pass
+        on = self.cfg.onconnect
+        logging.debug("onconnect is %s" % on)
+        if on: time.sleep(2) ; self._raw(on)
+        m = self.cfg.servermodes
+        if m:
+            time.sleep(2)
+            logging.debug("sending servermodes %s" % m)
+            self._raw("MODE %s %s" % (self.cfg.nick, m))
 
     def _resume(self, data, botname, reto=None):
         """ resume to server/port using nick. """
@@ -310,27 +318,26 @@ class Irc(BotBase):
             pass
         self.stopped = False
         try:
-            logging.info("%s - resume - file descriptor is %s" % (self.cfg.name, data['fd']))
             fd = int(data['fd'])
-        except (TypeError, ValueError):
+        except (KeyError, TypeError, ValueError):
             fd = None
             logging.error("%s - can't determine file descriptor" % self.cfg.name)
+            print data.tojson()
             return 0
+        logging.warn("resume - file descriptor is %s (%s)" % (fd, data.name))
         # create socket
-        if self.cfg.ipv6:
-            if fd: self.sock = socket.fromfd(fd , socket.AF_INET6, socket.SOCK_STREAM)
-            else: self.sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
-        else:
-            if fd: self.sock = socket.fromfd(fd, socket.AF_INET, socket.SOCK_STREAM)
-            else: self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self.sock.settimeout(30)
-        self.fsock = self.sock.makefile("r")
-        self.sock.setblocking(self.blocking)
+        if self.cfg.ipv6: self.oldsock = socket.fromfd(fd , socket.AF_INET6, socket.SOCK_STREAM)
+        else: self.oldsock = socket.fromfd(fd, socket.AF_INET, socket.SOCK_STREAM)
+        assert self.oldsock
+        self.oldsock.settimeout(30)
+        self.fsock = self.oldsock.makefile("r")
+        self.oldsock.setblocking(self.blocking)
         if self.blocking:
             socktimeout = self.cfg['socktimeout']
             if not socktimeout: socktimeout = 301.0
             else: socktimeout = float(socktimeout)
-            self.sock.settimeout(socktimeout)
+            self.oldsock.settimeout(socktimeout)
+        self.sock = self.oldsock
         self.nickchanged = 0
         self.connecting = False
         time.sleep(2)
@@ -340,13 +347,10 @@ class Irc(BotBase):
         self.connected = True
         self.reconnectcount = 0
         if reto: self.say(reto, 'rebooting done')
-        logging.warn("%s - rebooting done" % self.cfg.name)
+        logging.warn("rebooting done (%s)" % self.cfg.name)
         return True
 
     def outnocb(self, printto, what, how='msg', *args, **kwargs):
-        #if printto in self.nicks401:
-        #     logging.warn("%s - blocking %s" % (self.cfg.name, printto))
-        #     return
         what = fix_format(what)
         what = self.normalize(what)
         if 'socket' in repr(printto) and self.sock:
@@ -393,29 +397,20 @@ class Irc(BotBase):
             connect to server/port using nick .. connect can timeout so catch
             exception .. reconnect if enabled.
         """
-        try:
-            self._connect()
-            logging.info("%s - starting logon" % self.cfg.name)
-            self.logon()
-            time.sleep(1)
-            self.nickchanged = 0
-            self.reconnectcount = 0
-            self._onconnect()
-            self.connected = True
-            self.connecting = False
-            return True
-        except (socket.gaierror, socket.error), ex:
-            logging.error('%s - connecting error: %s' % (self.cfg.name, str(ex)))
-        except Exception, ex:
-            handle_exception()
-            logging.error('%s - connecting error: %s' % (self.cfg.name, str(ex)))
-        #self.connectok.set()
-        #if reconnect: self.reconnect()
-        return False
+        res = self._connect()
+        logging.info("%s - starting logon" % self.cfg.name)
+        self.logon()
+        time.sleep(1)
+        self.nickchanged = 0
+        self.reconnectcount = 0
+        self._onconnect()
+        self.connected = True
+        self.connecting = False
+        return res
 
     def shutdown(self):
         """ shutdown the bot. """
-        logging.warn('%s - shutdown' % self.cfg.name)
+        logging.warn('shutdown (%s)' % self.cfg.name)
         self.stopoutputloop = 1
         #self.close()
         self.connecting = False
@@ -481,7 +476,7 @@ class Irc(BotBase):
             self.state.save()
         except (KeyError, ValueError), ex:
             logging.error("error removing %s from joinedchannels: %s" % (channel, str(ex)))
-        if channel in self.cfg.channels: self.cfg.channels.remove(channel) ; self.cfg.save()
+        if self.cfg.channels and channel in self.cfg.channels: self.cfg.channels.remove(channel) ; self.cfg.save()
 
     def who(self, who):
         """ send who query. """
@@ -540,7 +535,7 @@ class Irc(BotBase):
 
     def quit(self, reason='http://jsonbot.googlecode.com'):
         """ send quit message. """
-        logging.warn('%s - sending quit - %s' % (self.cfg.name, reason))
+        logging.warn('sending quit - %s (%s)' % (reason, self.cfg.name))
         self._raw('QUIT :%s' % reason)
 
     def notice(self, printto, what):
@@ -599,18 +594,18 @@ class Irc(BotBase):
         self._raw("NICK %s" % randomnick)
         self.cfg.wantnick = self.cfg.nick
         self.cfg.nick = randomnick
-        logging.warn('%s - ALERT: nick %s already in use/unavailable .. using randomnick %s' % (self.cfg.name, nick, randomnick))
+        logging.warn('ALERT: nick %s already in use/unavailable .. using randomnick %s (%s)' % (nick, randomnick, randomnick))
         self.nickchanged = 1
 
     def handle_ping(self, ievent):
         """ send pong response. """
-        if not ievent.txt: logging.warn("no txt set") ; return
+        if not ievent.txt: logging.debug("no txt set") ; return
         self._raw('PONG :%s' % ievent.txt)
 
     def handle_001(self, ievent):
         """ we are connected.  """
+        time.sleep(1)
         self._onconnect()
-        time.sleep(2)
         self.connectok.set()
         self.connected = True
         self.whois(self.cfg.nick)
@@ -645,19 +640,21 @@ class Irc(BotBase):
     def handle_error(self, ievent):
         """ show error. """
         txt = ievent.txt
-        if txt.startswith('Closing'):
-            if "banned" in txt.lower(): logging.error("WE ARE BANNED !! - %s - %s" % (self.cfg.server, ievent.txt)) ; self.exit()
+        if 'Closing' in txt:
+            if "banned" in txt.lower() or "-lined" in txt.lower():
+                logging.error("WE ARE BANNED !! - %s - %s" % (self.cfg.server, txt))
+                self.exit()
             else: logging.error("%s - %s" % (self.cfg.name, txt))
         else: logging.error("%s - %s - %s" % (self.cfg.name.upper(), ", ".join(ievent.arguments[1:]), txt))
 
     def ping(self):
         """ ping the irc server. """
-        logging.warn('%s - sending ping' % self.cfg.name)
+        logging.debug('%s - sending ping' % self.cfg.name)
         try:
             self._raw('PING :%s' % self.cfg.server)
             return 1
         except Exception, ex:
-            logging.warn("%s - can't send ping: %s" % (self.cfg.name, str(ex)))
+            logging.warn("can't send ping: %s (%s)" % (str(ex), self.cfg.name))
             return 0
 
     def handle_401(self, ievent):
@@ -668,7 +665,7 @@ class Irc(BotBase):
         """ handle 700 .. encoding request of the server. """
         try:
             self.encoding = ievent.arguments[1]
-            logging.warn('%s - 700 encoding now is %s' % (self.cfg.name, self.encoding))
+            logging.warn('700 encoding now is %s (%s)' % (self.encoding, self.cfg.name))
         except:
             pass
 
