@@ -6,7 +6,7 @@
 
 ## jsb imports
 
-from jsb.lib.errors import CannotAuth
+from jsb.lib.errors import CannotAuth, CantLogon, NoUserProvided
 from jsb.lib.users import users
 from jsb.utils.exception import handle_exception
 from jsb.utils.trace import whichmodule
@@ -39,6 +39,8 @@ from errors import xmpperrors
 
 ## basic imports
 
+import time
+import urllib
 import copy
 import time
 import Queue
@@ -81,7 +83,7 @@ class SXMPPBot(XMLStream, BotBase):
     def __init__(self, cfg=None, usersin=None, plugs=None, jid=None, *args, **kwargs):
         BotBase.__init__(self, cfg, usersin, plugs, jid, *args, **kwargs)
         if not self.cfg: raise Exception("sxmpp - config is not set.")
-        if not self.cfg.user: raise Exception("sxmpp - user is not set.")
+        if not self.cfg.user: raise NoUserProvided("please make sure the user config variable is set in %s (or use -u)"  % cfg.cfile)
         try: self.cfg.username, self.cfg.host = self.cfg.user.split('@')
         except (ValueError, TypeError): raise Exception("%s - user not set - %s" % (self.cfg.name, str(self.cfg)))
         XMLStream.__init__(self, self.cfg.name)   
@@ -99,7 +101,7 @@ class SXMPPBot(XMLStream, BotBase):
         if self.state and not self.state.data.ratelimit: self.state.data.ratelimit = 0.02
         try: self.cfg.port = int(self.cfg.port)
         except (ValueError, TypeError): self.cfg.port = 5222
-        logging.warn("%s - user is %s" % (self.cfg.name, self.cfg.user))
+        logging.debug("%s - user is %s" % (self.cfg.name, self.cfg.user))
         
     def _resumedata(self):
         """ return data needed for resuming. """
@@ -159,12 +161,15 @@ class SXMPPBot(XMLStream, BotBase):
     def connect(self, reconnect=False):
         """ connect the xmpp server. """
         if not XMLStream.doconnect(self): return False
+        self.connection = None
         iq = self.makeready()
         if not iq:
             logging.error('%s - connect to %s:%s (%s) failed' % (self.cfg.name, self.cfg.host, self.cfg.port, self.cfg.server))
             return
         self.logon(self.cfg.user, self.cfg.password, iq)
-        self._raw("<presence/>")
+        time.sleep(1)
+        self.sendpresence()
+        #self._raw("<presence/>")
         start_new_thread(self._keepalive, ())
         if self.cfg.keepchannelsalive: start_new_thread(self._keepchannelsalive, ())
         if self.cfg.doroster: self.requestroster()
@@ -187,14 +192,15 @@ class SXMPPBot(XMLStream, BotBase):
                 except Exception, ex: self.exit() ; raise
                 time.sleep(5)
                 iq = self.auth_sasl(user, password, iq, False)
-            else: logging.error("stopping .. you can try to use --register to register the bot with the server") ; return
+            else: logging.error("stopping .. you can try to use --register to register the bot with the server") ; raise CantLogon(user)
         XMLStream.logon(self)
+        return True
 
     def register(self, jid, password):
         """ register the jid to the server. """
         try: resource = jid.split("/")[1]
         except IndexError: resource = "jsb"
-        logging.warn('%s - registering %s' % (self.cfg.name, jid))
+        logging.debug('%s - registering %s' % (self.cfg.name, jid))
         iq = self.waiter("""<iq type='get'><query xmlns='jabber:iq:register'/></iq>""")
         if not iq:
             logging.error("%s - unable to register" % self.cfg.name)
@@ -204,7 +210,7 @@ class SXMPPBot(XMLStream, BotBase):
             logging.warn('%s - register FAILED - %s' % (self.cfg.name, iq.error))
             if not iq.error.code: logging.error("%s - can't determine error code" % self.cfg.name) ; return False
             if iq.error.code == "405": logging.error("%s - this server doesn't allow registration by the bot, you need to create an account for it yourself" % self.cfg.name)
-            elif iq.error.code == "500": logging.error("%s - %s - %s" % (self.cfg.name, iq.error.code, iq.error.text))
+            elif iq.error.code == "500": logging.error("%s - %s - %s" % (self.cfg.name, iq.error.code, iq.error.text.data))
             else: logging.error("%s - %s" % (self.cfg.name, xmpperrors[iq.error.code]))
             self.error = iq.error
             raise Exception(iq.error)
@@ -222,10 +228,10 @@ class SXMPPBot(XMLStream, BotBase):
     def outnocb(self, printto, txt, how=None, event=None, html=False, isrelayed=False, *args, **kwargs):
         """ output txt to bot. """
         if printto and printto in self.state['joinedchannels']: outtype = 'groupchat'
-        else: outtype = "chat"
+        else: outtype = (event and event.type) or "chat"
         target = printto
-        if not html: 
-            txt = self.normalize(txt)
+        txt = self.normalize(txt)
+        #txt = stripcolor(txt)
         repl = Message(event)
         repl.to = target
         repl.type = outtype
@@ -256,8 +262,9 @@ class SXMPPBot(XMLStream, BotBase):
             nm = Message(m)
             callbacks.check(self, nm)
             return
-        if data.get('x').xmlns == 'jabber:x:delay':
-            logging.debug("%s - ignoring delayed message" % self.cfg.name)
+        xmlns = data.get('x').xmlns or data.get("delay").xmlns
+        if xmlns and (xmlns == 'jabber:x:delay' or xmlns == 'urn:xmpp:delay'):
+            logging.warn("%s - ignoring delayed message" % self.cfg.name)
             return
         if m.isresponse:
             logging.debug("%s - message is a response" % self.cfg.name)
@@ -275,12 +282,11 @@ class SXMPPBot(XMLStream, BotBase):
             if utarget: m.userhost = m.jid = m.auth = stripped(utarget)
             else: m.userhost = m.jid
         if m.msg: m.userhost = stripped(m.userhost)
-        logging.warn("using %s as userhost" % m.userhost)
+        logging.debug("using %s as userhost" % m.userhost)
         try:
             if m.type == 'error':
-                if m.code:
-                    logging.error('%s - error - %s' % (self.cfg.name, str(m)))
-                self.errorHandler(m)
+                if m.code: self.errorHandler(m)
+            else: m.nodispatch = False
         except Exception, ex:
             handle_exception()
         self.put(m)
@@ -299,7 +305,7 @@ class SXMPPBot(XMLStream, BotBase):
         frm = p.fromm
         nickk = ""
         nick = p.nick
-        if self.cfg.user in p.userhost: return 0
+        if self.cfg.user in frm: self.pongcheck = True 
         if nick: 
             self.userhosts[nick] = stripped(frm)
             nickk = nick
@@ -315,7 +321,7 @@ class SXMPPBot(XMLStream, BotBase):
                 self.jids[channel] = {}
             self.jids[channel][nickk] = jid
             self.userhosts[nickk] = stripped(jid)
-            logging.warn('%s - setting jid of %s (%s) to %s' % (self.cfg.name, nickk, channel, self.userhosts[nickk]))
+            logging.debug('%s - setting jid of %s (%s) to %s' % (self.cfg.name, nickk, channel, self.userhosts[nickk]))
         if p.type == 'subscribe':
             pres = Presence({'to': p.fromm, 'type': 'subscribed'})
             self.send(pres)
@@ -437,12 +443,13 @@ class SXMPPBot(XMLStream, BotBase):
     def join(self, channel, password=None, nick=None):
         """ join conference. """
         if channel.startswith("#"): return
+        logging.warn("joining %s" % channel)
         try:
             if not nick: nick = channel.split('/')[1]
         except IndexError: nick = self.cfg.nick or "jsonbot"
         channel = channel.split('/')[0]
         q = Queue.Queue()
-        presence = Presence({'to': channel + '/' + nick})
+        presence = Presence({'to': channel + '/' + nick, "xmlns": 'http://jabber.org/protocol/muc' })
         if password:
              presence.x.password = password             
         self.send(presence)
@@ -504,7 +511,6 @@ class SXMPPBot(XMLStream, BotBase):
         self.put(msg)
 
     def normalize(self, what):
-        #what = cgi.escape(what)
         what = stripcolor(what)
         what = what.replace("\002", "")
         what = what.replace("\003", "")
@@ -521,16 +527,23 @@ class SXMPPBot(XMLStream, BotBase):
         what = what.replace("&lt;/i&gt;", "")
         return what
 
-    def doreconnect(self, first=False):
+    #def doreconnect(self, first=False):
         """ reconnect to the server. """
-        botjid = self.cfg.user
+        """botjid = self.cfg.user
         if not first:
-            newbot = getfleet().makebot('sxmpp', cfg.name, config=self.cfg, showerror=True)
+            newbot = getfleet().makebot('sxmpp', self.cfg.name, config=self.cfg, showerror=True)
             if not newbot: raise Exception("can't make bot %s with config %s" % (self.cfg.name, self.cfg.tojson()))
             newbot.reconnectcount = self.reconnectcount
         else: newbot = self
         newbot.start(connect=True)
         newbot.joinchannels()
         if not first: getfleet().replace(botjid, newbot)
-        return True
+        """
+    #    self.connect()
+    #    return True
 
+    def pingcheck(self):
+        self.pongcheck = False
+        self.sendpresence()
+        time.sleep(5)
+        return self.pongcheck
