@@ -9,7 +9,7 @@
 
 ## jsb imports
 
-from jsb.lib.errors import CannotAuth
+from jsb.lib.errors import CannotAuth, StreamError
 from jsb.lib.eventbase import EventBase
 from jsb.lib.config import Config
 from jsb.utils.generic import toenc, jabberstrip, fromenc
@@ -96,17 +96,17 @@ class XMLStream(NodeBuilder):
 
     def handle_success(self, data):
         """ default stream handler. """
-        logging.info("%s - BINGO !! is %s" % (self.cfg.name, data.dump()))
+        logging.info("%s - BINGO !! is %s" % (self.cfg.name, data.tojson()))
 
     def handle_failure(self, data):
         """ default stream handler. """
-        logging.info("%s - failure is %s" % (self.cfg.name, data.dump()))
+        logging.error("%s - failure is %s" % (self.cfg.name, data.tojson()))
         self.failure = data
         self.stopped = True
 
     def handle_challenge(self, data):
         """ default stream handler. """
-        logging.info("%s - challenge is %s" % (self.cfg.name, data.dump()))
+        logging.debug("%s - challenge is %s" % (self.cfg.name, data.tojson()))
 
     def handle_proceed(self, data):
         """ default stream handler. """
@@ -114,19 +114,18 @@ class XMLStream(NodeBuilder):
 
     def handle_stream(self, data):
         """ default stream handler. """
-        logging.info("%s - stream - %s" % (self.cfg.name, data.dump()))
+        logging.debug("%s - stream - %s" % (self.cfg.name, data.tojson()))
         if data and data.id: self.streamid = data.id
 
     def handle_streamend(self, data):
         """ default stream handler. """
         logging.warn("%s - stream END - %s" % (self.cfg.name, data))
-        self.stopped = True
 
     def handle_streamerror(self, data):
         """ default stream error handler. """
-        logging.error("%s - STREAMERROR - %s" % (self.cfg.name, data.dump()))
-        self.stopped = True
- 
+        logging.error("%s - STREAMERROR - %s" % (self.cfg.name, data.tojson()))
+        #raise StreamError(data.tojson())
+         
     def handle_streamfeatures(self, data):
         """ default stream features handler. """
         logging.debug("%s - STREAMFEATURES: %s" % (self.cfg.name, LazyDict(data).dump()))
@@ -187,7 +186,7 @@ class XMLStream(NodeBuilder):
             try:
                 data = jabberstrip(fromenc(self.connection.read()))
                 if self.stopped or self.stopreadloop: break
-                logging.warn(u"< %s" % data)
+                logging.info(u"< %s (%s)" % (data, len(data)))
                 if data.endswith("</stream:stream>"):
                     logging.error("%s - end of stream detected" % self.cfg.name)
                     self.error = "streamend"
@@ -230,7 +229,7 @@ class XMLStream(NodeBuilder):
                 self.error = str(ex)
                 self.disconnectHandler(ex)
                 break
-        logging.warn('%s - stopping readloop .. %s' % (self.cfg.name, self.error or 'error not set'))
+        logging.debug('%s - stopping readloop .. %s' % (self.cfg.name, self.error or 'error not set'))
 
     @outlocked
     def _raw(self, stanza):
@@ -248,11 +247,14 @@ class XMLStream(NodeBuilder):
                 return
             start = what[:3]
             if start in ['<st', '<me', '<pr', '<iq', "<au", "<re", "<fa", "<ab"]:
-                logging.warn(u"> %s" % what)
-                try: self.connection.send(what + u"\r\n")
-                except AttributeError:
-                    try: self.connection.write(what)
-                    except AttributeError: self.sock.send(what)
+                if start == "<pr": logging.debug(u"> %s" % what)
+                else: logging.info(u"> %s" % what)
+                if not self.connection: self.sock.send(what)
+                else:
+                    try: self.connection.send(what + u"\r\n")
+                    except AttributeError:
+                        try: self.connection.write(what)
+                        except AttributeError: self.sock.send(what)
             else: logging.error('%s - invalid stanza: %s' % (self.cfg.name, what))
             if self.cfg.sleeptime: time.sleep(self.cfg.sleeptime)
             else: time.sleep(0.01)
@@ -275,7 +277,7 @@ class XMLStream(NodeBuilder):
                 try: result = self.connection.read()
                 except AttributeError: result = self.sock.recv(1500)
                 if not result: time.sleep(0.1) ; continue
-                logging.info("%s - %s" %  (self.cfg.name, result))
+                logging.info("< %s (%s)" %  (result, self.cfg.name))
                 res = self.loop_one(result)
                 if self.stopped or self.failure: break
                 if not find: break
@@ -292,25 +294,21 @@ class XMLStream(NodeBuilder):
             import dns.resolver
         except: pass
         else:
-            # taken from SleekXMPP. see https://github.com/fritzy/SleekXMPP
             try:
                 xmpp_srv = "_xmpp-client._tcp.%s" % (self.cfg.server or self.cfg.host)
                 answers = dns.resolver.query(xmpp_srv, dns.rdatatype.SRV)
             except ImportError:
                 loging.warn("resolver is not available.")
             except (dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
-                logging.warn("no appropriate SRV record found.")
+                logging.info("no appropriate SRV record found.")
             else:
-                # Pick a random server, weighted by priority.
                 addresses = {}
                 intmax = 0
                 for answer in answers:
-                    print dir(answer)
                     try:
                         intmax += answer.priority
                         addresses[intmax] = (answer.target.to_text()[:-1], answer.port)
-                    except Exception, ex: logging.warn("%s - skipping %s: %s" % (self.cfg.name, str(answer), str(ex)))
-                #python3 returns a generator for dictionary keys
+                    except Exception, ex: logging.debug("%s - skipping %s: %s" % (self.cfg.name, str(answer), str(ex)))
                 priorities = [x for x in addresses.keys()]
                 priorities.sort()
 
@@ -324,11 +322,11 @@ class XMLStream(NodeBuilder):
         logging.debug("%s - TARGET is %s" % (self.cfg.name, target))
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self.sock.setblocking(0)
-        self.sock.settimeout(10)
-        logging.warn("%s - connecting to %s" % (self.cfg.name, target))
+        self.sock.settimeout(30)
+        logging.warn("connecting to %s (%s)" % (target, self.cfg.name))
         self.sock.connect(target)
         self.target = target
-        logging.warn("%s - connected to server." % self.cfg.name)
+        logging.warn("connected! trying to login as %s (%s)" % (self.cfg.user, self.cfg.name))
         return True
 
     def makeready(self):
@@ -352,7 +350,7 @@ class XMLStream(NodeBuilder):
 
     def auth(self, jid, password, iq=None, initstream=True):
         """ auth against the xmpp server. """
-        logging.warn('%s - authing %s' % (self.cfg.name, jid))
+        logging.warn('authing %s (%s)' % (jid, self.cfg.name))
         (name, host) = jid.split('@')
         rsrc = self.cfg['resource'] or self.cfg['resource'] or 'jsb';
         if self.cfg.nosasl: self.auth_nosasl(jid, password, iq)
@@ -375,7 +373,7 @@ class XMLStream(NodeBuilder):
         self.features.sort()
         for method in self.features:
             logging.debug("method is %s" % method)
-            if method not in ["DIGEST-MD5", "PLAIN"]: logging.warn("skipping %s" % method) ; continue
+            if method not in ["DIGEST-MD5", "PLAIN"]: logging.debug("skipping %s" % method) ; continue
             #if method not in ["PLAIN"]: logging.warn("skipping %s" % method) ; continue
             try:
                 meth = getattr(self, "auth_%s" % method.replace("-", "_").lower())
@@ -388,7 +386,7 @@ class XMLStream(NodeBuilder):
 
     def auth_nosasl(self, jid, password, iq=None):
         """ auth against the xmpp server. """
-        logging.warn('%s - authing %s' % (self.cfg.name, jid))
+        logging.debug('%s - authing %s' % (self.cfg.name, jid))
         (name, host) = jid.split('@')
         rsrc = self.cfg['resource'] or self.cfg['resource'] or 'jsb';
         iq = self._raw('<stream:stream to="%s" xmlns="jabber:client" xmlns:stream="http://etherx.jabber.org/streams">' % self.cfg.user.split('@')[1])
@@ -409,7 +407,7 @@ class XMLStream(NodeBuilder):
         resp = self.waiter("""<auth xmlns='urn:ietf:params:xml:ns:xmpp-sasl' mechanism='DIGEST-MD5'/>""", "challenge")
         challenge = re.findall(">(.*?)</challenge>", resp.orig)
         if not challenge: logging.error("%s - can't find challenge" % self.cfg.name)
-        else: self.challenge = challenge[0] ; logging.info("challenge is %s" % self.challenge)
+        else: self.challenge = challenge[0] ; logging.debug("challenge is %s" % self.challenge)
         response = makeresp("xmpp/%s" % host, host, name, password, self.challenge)
         resp = self.waiter("<response xmlns='urn:ietf:params:xml:ns:xmpp-sasl'>%s</response>" % response.replace("\n", ""))
         if self.failure: raise CannotAuth(self.failure.orig)
@@ -421,9 +419,10 @@ class XMLStream(NodeBuilder):
 
     def auth_plain(self, jid, password, iq=None):
         (name, host) = jid.split('@')
-        if not self.cfg.openfire: jid = jid.lower() + "/" + (self.cfg.resource or "bot")
+        if "hipchat" in host: jid = name
+        elif not self.cfg.openfire: jid = jid.lower() + "/" + (self.cfg.resource or "bot")
+        logging.debug("jid is %s" % jid)
         rsrc = self.cfg['resource'] or self.cfg['resource'] or 'jsb';
-        #if self.streamid: password = password + self.streamid ; logging.warn("added %s to password" % self.streamid)
         if sys.version_info < (3, 0):
             user = bytes(jid)
             passw = bytes(password)
@@ -441,13 +440,11 @@ class XMLStream(NodeBuilder):
         return resp
 
     def init_tls(self):
-        if self.stopped: return
         logging.info("%s - starting TLS" % self.cfg.name)
         return self.waiter('<starttls xmlns="urn:ietf:params:xml:ns:xmpp-tls"/>')
 
     def dossl(self):
         """ enable ssl on the socket. """
-        if self.stopped: return
         try: import ssl ; self.connection = ssl.wrap_socket(self.sock)
         except ImportError: self.connection = socket.ssl(self.sock)
         if self.connection: return True
@@ -549,5 +546,6 @@ class XMLStream(NodeBuilder):
 
     def disconnectHandler(self, ex):
         """ handler called on disconnect. """
-        self.stopped = True
-        logging.warn('%s - disconnected: %s' % (self.cfg.name, str(ex)))
+        logging.warn('disconnected: %s (%s)' % (str(ex), self.cfg.name))
+        time.sleep(5)
+        self.reconnect()
\ No newline at end of file
