@@ -78,7 +78,7 @@ class BotBase(LazyDict):
 
     """ base class for all bots. """
 
-    def __init__(self, cfg=None, usersin=None, plugs=None, botname=None, nick=None, *args, **kwargs):
+    def __init__(self, cfg=None, usersin=None, plugs=None, botname=None, nick=None, bottype=None, *args, **kwargs):
         logging.debug("type is %s" % str(type(self)))
         if cfg: self.cfg = cfg ; botname = botname or self.cfg.name
         if not botname: botname = u"default-%s" % str(type(self)).split('.')[-1][:-2]
@@ -87,21 +87,17 @@ class BotBase(LazyDict):
         if not self.cfg: self.cfg = Config(self.fleetdir + os.sep + u'config')
         self.cfg.name = botname or self.cfg.name
         if not self.cfg.name: raise Exception("name is not set in %s config file" % self.fleetdir)
-        logging.info("name is %s" % self.cfg.name)
+        logging.debug("name is %s" % self.cfg.name)
         LazyDict.__init__(self)
         logging.debug("created bot with config %s" % self.cfg.tojson(full=True))
         self.ecounter = 0
         self.ignore = []
         self.ids = []
-        self.started = False
         self.aliases = getaliases()
-        self.curevent = None
         self.reconnectcount = 0
-        self.stopped = False
         self.plugs = coreplugs
         self.gatekeeper = GateKeeper(self.cfg.name)
         self.gatekeeper.allow(self.user or self.jid or self.cfg.server or self.cfg.name)
-        self.closed = False
         try:
             import waveapi
             self.isgae = True
@@ -110,21 +106,23 @@ class BotBase(LazyDict):
             self.isgae = False
             logging.debug("bot is a shell bot (%s)" % self.cfg.name)
         self.starttime = time.time()
-        self.type = "base"
+        self.type = bottype or "base"
         self.status = "init"
         self.networkname = self.cfg.networkname or self.cfg.name or ""
         from jsb.lib.datadir import getdatadir
         datadir = getdatadir()
         self.datadir = datadir + os.sep + self.fleetdir
+        self.maincfg = getmainconfig()
         self.owner = self.cfg.owner
         if not self.owner:
             logging.debug(u"owner is not set in %s - using mainconfig" % self.cfg.cfile)
-            self.owner = getmainconfig().owner
-        self.users = getusers()
+            self.owner = self.maincfg.owner
+        self.users = usersin or getusers()
         logging.debug(u"owner is %s" % self.owner)
         self.users.make_owner(self.owner)
         self.outcache = outcache
         self.userhosts = LazyDict()
+        self.nicks = LazyDict()
         self.connectok = threading.Event()
         self.reconnectcount = 0
         self.cfg.nick = nick or self.cfg.nick or u'jsb'
@@ -132,9 +130,6 @@ class BotBase(LazyDict):
             if not os.isdir(self.datadir): os.mkdir(self.datadir)
         except: pass
         self.setstate()
-        self.stopreadloop = False
-        self.stopoutloop = False
-        self.stopeventloop = False
         self.outputlock = thread.allocate_lock()
         try:
             self.outqueue = Queue.PriorityQueue()
@@ -146,22 +141,7 @@ class BotBase(LazyDict):
         self.cmndperms = getcmndperms()
         self.outputmorphs = outputmorphs
         self.inputmorphs = inputmorphs
-        if not self.isgae:
-            defaultrunner.start()
-            callbackrunner.start()
-            waitrunner.start()
-            tickloop.start(self)
-
-    #def __setattr__(self, a ,b):
-    #    if self.cfg and a == "cfg": raise Exception("attempt to overwrite config")
-    #    else: self[a] = b
-
-    #def __deepcopy__(self, a):
-    #    """ deepcopy an event. """  
-    #    logging.debug("cpy - %s" % type(self))
-    #    bot = bot_factory.create(self.type, self.cfg)
-    #    bot.users = getusers()
-    #    return bot
+        if not self.isgae: tickloop.start(self)
 
     def copyin(self, data):
         self.update(data)
@@ -171,40 +151,43 @@ class BotBase(LazyDict):
 
     def _resumedata(self):
         """ return data needed for resuming. """
-        data = self.cfg
-        try: data.fd = self.sock.fileno()
-        except: pass
-        return {self.cfg.name: data}
+        try: self.cfg.fd = self.oldsock.fileno()
+        except AttributeError: logging.warn("no oldsock found for %s" % self.cfg.name)
+        return {self.cfg.name: dict(self.cfg)}
 
-    def benice(self, event=None):
+    def benice(self, event=None, sleep=0.005):
         if self.server and self.server.io_loop:
             logging.debug("i'm being nice")
-            if event and self.server and event.handler: self.server.io_loop.add_callback(event.handler.async_callback(lambda: time.sleep(0.001)))
-            elif self.server: self.server.io_loop.add_callback(lambda: time.sleep(0.001))
-        time.sleep(0.001)
+            if event and self.server and event.handler: self.server.io_loop.add_callback(event.handler.async_callback(lambda: time.sleep(sleep)))
+            elif self.server: self.server.io_loop.add_callback(lambda: time.sleep(sleep))
+        time.sleep(sleep)
 
-    def enable(self, modname):
+    def do_enable(self, modname):
         """ enable plugin given its modulename. """
         try: self.cfg.blacklist and self.cfg.blacklist.remove(modname)
         except ValueError: pass           
         if self.cfg.loadlist and modname not in self.cfg.loadlist: self.cfg.loadlist.append(modname)
         self.cfg.save()
 
-    def disable(self, modname):
+    def do_disable(self, modname):
         """ disable plugin given its modulename. """
         if self.cfg.blacklist and modname not in self.cfg.blacklist: self.cfg.blacklist.append(modname)
         if self.cfg.loadlist and modname in self.cfg.loadlist: self.cfg.loadlist.remove(modname)
         self.cfg.save()
 
-    def put(self, event):
+    #@locked
+    def put(self, event, direct=False):
         """ put an event on the worker queue. """
-        if not event: return event
-        if self.isgae:
+        if direct: self.doevent(event)
+        elif self.isgae:
             from jsb.drivers.gae.tasks import start_botevent
             start_botevent(self, event, event.speed)
         else:
-            logging.info("%s - putted event %s" % (self.cfg.name, event.tojson()))
-            self.input(event.speed, event)
+            if event:
+                 logging.debug("putted event on %s" % self.cfg.name)
+                 self.ecounter += 1
+                 self.input(event.speed, event)
+            else: self.input(0, None)
         return event
 
     def broadcast(self, txt):
@@ -214,27 +197,29 @@ class BotBase(LazyDict):
 
     def _eventloop(self):
         """ output loop. """
-        logging.info('%s - starting eventloop' % self.cfg.name)
+        logging.debug('%s - starting eventloop' % self.cfg.name)
         self.stopeventloop = 0
         while not self.stopped and not self.stopeventloop:
             try:
                 res = self.eventqueue.get()
-                if not res: continue
+                if not res: break
                 (prio, event) = res
-                logging.info("%s - eventloop - %s - %s" % (self.cfg.name, event.cbtype, event.userhost)) 
+                if not event: break
+                logging.debug("%s - eventloop - %s - %s" % (self.cfg.name, event.cbtype, event.userhost)) 
+                event.speed = prio
                 self.doevent(event)
-            except Queue.Empty: time.sleep(0.1) ; continue
+                self.benice()
+            except Queue.Empty: time.sleep(0.01) ; continue
             except Exception, ex: handle_exception() ; logging.warn("error in eventloop: %s" % str(ex))
-        logging.info('%s - stopping eventloop' % self.cfg.name)
+        logging.debug('%s - stopping eventloop' % self.cfg.name)
 
     def input(self, prio, event):
         """ put output onto one of the output queues. """
-        if not event: logging.error("no event as input") ; return
         self.eventqueue.put(("%s-%s" % (prio, self.ecounter), event))
 
     def _outloop(self):
         """ output loop. """
-        logging.info('%s - starting output loop' % self.cfg.name)
+        logging.debug('%s - starting output loop' % self.cfg.name)
         self.stopoutloop = 0
         while not self.stopped and not self.stopoutloop:
             try:
@@ -246,7 +231,18 @@ class BotBase(LazyDict):
                 self.out(*res)
             except Queue.Empty: time.sleep(0.1) ; continue
             except Exception, ex: handle_exception()
-        logging.info('%s - stopping output loop' % self.cfg.name)
+        logging.debug('%s - stopping output loop' % self.cfg.name)
+
+    def _pingloop(self):
+        """ output loop. """
+        logging.debug('%s - starting ping loop' % self.cfg.name)
+        time.sleep(5)
+        while not self.stopped:
+            try:
+                if self.status != "start" and not self.pingcheck(): self.reconnect() ; break
+            except Exception, ex: logging.error(str(ex)) ; self.reconnect() ; break
+            time.sleep(self.cfg.pingsleep or 60)
+        logging.debug('%s - stopping ping loop' % self.cfg.name)
 
     def putonqueue(self, nr, *args):
         """ put output onto one of the output queues. """
@@ -297,24 +293,43 @@ class BotBase(LazyDict):
                 handle_exception()
             time.sleep(3)
 
+    def boot(self):
+        logging.warn("booting %s bot" % self.cfg.name)
+        if not self.cfg.type: self.cfg.type = self.type ; self.cfg.save()
+        fleet = getfleet()
+        fleet.addbot(self)
+        fleet.addnametype(self.cfg.name, self.type)
+        while 1:
+            try:
+                #self.exit(close=False, save=False)
+                self.started = False
+                if self.start(): break
+            except Exception, ex:
+                logging.error(str(ex))
+                logging.error("sleeping 15 seconds")
+                time.sleep(15)       
+
     def start(self, connect=True, join=True):
         """ start the mainloop of the bot. """
         if self.started: logging.warn("%s - already started" % self.cfg.name) ; return
-        self.started = True
-        self.status == "running"
+        self.stopped = False
+        self.stopreadloop = False
+        self.stopoutloop = False
+        self.status = "start"
         if not self.isgae:
-            if connect: self.connect()
-            start_new_thread(self._outloop, ())
             start_new_thread(self._eventloop, ())
-            start_new_thread(self._readloop, ())
             if connect:
-                self.connectok.wait(300)
-                if self.stopped: logging.warn("bot is stopped") ; return
+                if not self.connect() : return False
+                start_new_thread(self._readloop, ())
+                start_new_thread(self._outloop, ())
+                self.connectok.wait()
+                if self.stopped: logging.warn("bot is stopped") ; return True
                 if self.connectok.isSet():
                     logging.warn('%s - logged on !' % self.cfg.name)
                     if join: start_new_thread(self.joinchannels, ())
                 elif self.type not in ["console", "base"]: logging.warn("%s - failed to logon - connectok is not set" % self.cfg.name)
-        self.status == "running"
+        self.status == "started"
+        self.started = True
         self.dostart(self.cfg.name, self.type)
         return True
 
@@ -324,13 +339,11 @@ class BotBase(LazyDict):
         event.nodispatch = True
         event.forwarded = True
         event.dontbind = True
-        logging.info("======== start handling REMOTE event ========")
         event.prepare(self)
         self.status = "callback"
         starttime = time.time()
         msg = "%s - %s - %s - %s" % (self.cfg.name, event.auth, event.how, event.cbtype)
-        if event.how == "background": logging.debug(msg)
-        else: logging.info(msg)
+        logging.warn(msg)
         try: logging.debug("remote - %s" % event.dump())
         except: pass
         if self.closed:
@@ -352,20 +365,10 @@ class BotBase(LazyDict):
         try:
             if event.isremote(): self.doremote(event) ; return
             if event.type == "groupchat" and event.fromm in self.ids:
-                logging.warn("%s - receiving groupchat from self (%s)" % (self.cfg.name, event.fromm))
+                logging.debug("%s - receiving groupchat from self (%s)" % (self.cfg.name, event.fromm))
                 return
             event.txt = self.inputmorphs.do(fromenc(event.txt, self.encoding), event)
         except UnicodeDecodeError: logging.warn("%s - got decode error in input .. ingoring" % self.cfg.name) ; return
-        logtxt = "===> %s event" % event.cbtype
-        if event.cbtype in ['NOTICE']: logging.warn("%s - %s" % (event.nick, event.txt))
-        else:
-            try:
-                int(event.cbtype)
-                logging.debug(logtxt)
-            except (ValueError, TypeError):
-                if event.cbtype in ['PING',] or event.how == "background": 
-                    logging.debug(logtxt)
-                else: logging.warn(logtxt)
         event.bind(self)
         try: logging.debug("%s - event dump: %s" % (self.cfg.name, event.dump()))
         except: pass
@@ -377,7 +380,7 @@ class BotBase(LazyDict):
             logging.debug("%s - event is done .. ignoring" % self.cfg.name)
             return
         if event.msg or event.isdcc: event.speed = 2
-        e1 = event
+        e1 = cpy(event)
         first_callbacks.check(self, e1)
         if not e1.stop: 
             callbacks.check(self, e1)
@@ -390,9 +393,10 @@ class BotBase(LazyDict):
         """ check if provided userhost belongs to an owner. """
         if self.cfg and self.cfg.owner:
             if userhost in self.cfg.owner: return True
+        logging.warn("failed ownercheck for %s" % userhost)
         return False
 
-    def exit(self, stop=True):
+    def exit(self, stop=True, close=True, save=True, quit=False):
         """ exit the bot. """ 
         logging.warn("%s - exit" % self.cfg.name)
         if stop:
@@ -400,11 +404,15 @@ class BotBase(LazyDict):
             self.stopreadloop = True  
             self.connected = False
             self.started = False
+        if close:
             self.putonqueue(1, None, "")
             self.put(None)
-        self.shutdown()
-        self.save()
-
+            self.shutdown()
+        save and self.save()
+        fleet = getfleet()
+        fleet.remove(self)
+        if quit and not fleet.bots: globalshutdown()
+        
     def _raw(self, txt, *args, **kwargs):
         """ override this. outnocb() is used more though. """ 
         logging.debug(u"%s - out - %s" % (self.cfg.name, txt))
@@ -418,14 +426,15 @@ class BotBase(LazyDict):
         res1, nritems = self.less(origin or printto, txt, nr+extend)
         return res1
 
-    def out(self, printto, txt, how="msg", event=None, origin=None, html=False, *args, **kwargs):
+    def out(self, printto, txt, how="msg", event=None, origin=None, *args, **kwargs):
         """ output method with OUTPUT event generated. """
-        self.outnocb(printto, txt, how, event=event, origin=origin, html=html, *args, **kwargs)
         self.outmonitor(origin, printto, txt, event=event)
+        self.outnocb(printto, txt, how, event=event, origin=origin, *args, **kwargs)
+        if event: event.ready()
 
     write = out
 
-    def outnocb(self, printto, txt, how="msg", event=None, origin=None, html=False, *args, **kwargs):
+    def outnocb(self, printto, txt, how="msg", event=None, origin=None, *args, **kwargs):
         """ output function without callbacks called.. override this in your driver. """
         self._raw(txt)
 
@@ -433,17 +442,15 @@ class BotBase(LazyDict):
 
     def say(self, channel, txt, result=[], how="msg", event=None, nr=375, extend=0, dot=", ", showall=False, *args, **kwargs):
         """ default method to send txt from the bot to a user/channel/jid/conference etc. """
-        if event: event.busy.appendleft(self.cfg.name)
         logging.warn("saying to %s" % channel)
         if event:
+            #event.busy.append(event.usercmnd)
             if event.userhost in self.ignore: logging.warn("%s - ignore on %s - no output done" % (self.cfg.name, event.userhost)) ; return
-            if event.pipelined:
-                logging.warn("pipelined .. only pushing to outqueue")
-                event.outqueue.append(txt)
-                for i in result: event.outqueue.append(i)
-                return
             if event.how == "msg" and self.type == "irc": target = event.nick
             else: target = channel
+            if event.pipelined:
+                for i in result: event.outqueue.append(i)
+                return
         else: target = channel
         if showall or (event and event.showall): txt = self.makeresponse(txt, result, dot, *args, **kwargs)
         else: txt = self.makeoutput(channel, txt, result, nr, extend, dot, origin=target, *args, **kwargs)
@@ -451,18 +458,17 @@ class BotBase(LazyDict):
             txt = decode_html_entities(txt)
             if event:
                 event.nrout += 1
-                if event.pipelined: return
                 if event.displayname: txt = "[%s] %s" % (event.displayname, txt)
-                for i in result: event.outqueue.append(i)
+                if result:
+                    for i in result: event.outqueue.append(i)
                 event.resqueue.append(txt)
-                if event.path != None and not self.cfg.name in event.path: event.path.append(self.cfg.name)
-                event.ready()
-                if event.nooutput: return
-            else: logging.warn("no putting txt on queues")
+                if event.nooutput: event.ready() ; return
+            else: logging.info("not putting txt on queues")
             txt = self.outputmorphs.do(txt, event)
             self.out(target, txt, how, event=event, origin=target, *args, **kwargs)
 
     def saynocb(self, channel, txt, result=[], how="msg", event=None, nr=375, extend=0, dot=", ", showall=False, *args, **kwargs):
+        logging.warn("saying to %s (without callbacks)" % channel)
         txt = self.makeoutput(channel, txt, result, nr, extend, dot, showall=showall, *args, **kwargs)
         if txt:
             if event:
@@ -485,69 +491,33 @@ class BotBase(LazyDict):
         res = txtlist[0]
         length = len(txtlist)
         if length > 1:
-            logging.info("addding %s lines to %s outcache (less)" % (len(txtlist), printto))
+            logging.debug("addding %s lines to %s outcache (less)" % (len(txtlist), printto))
             outcache.set(u"%s-%s" % (self.cfg.name, printto), txtlist[1:])
             res += "<b> - %s more</b>" % (length - 1) 
         return [res, length]
 
-    def join(self, channel, password, *args, **kwargs):
-        """ join a channel. """
-        pass
 
-    def part(self, channel, *args, **kwargs):
-        """ leave a channel. """
-        pass
-
-    def action(self, channel, txt, event=None, *args, **kwargs):
-        """ send action to channel. """
-        pass
-
-    def doop(self, channel, who):
-        """ give nick ops. """
-        pass
-
-    def reconnect(self, start=False):
+    def reconnect(self, start=False, close=False):
         """ reconnect to the server. """
         if self.stopped: logging.warn("%s - bot is stopped .. not reconnecting" % self.cfg.name) ; return
+        #self.reconnectcount = 0
         time.sleep(2)
         while 1:
-            try:
-                if not start: self.exit()
-                if self.doreconnect(start): break
-            except Exception, ex: 
-                handle_exception()
             self.reconnectcount += 1
             sleepsec = self.reconnectcount * 5
             if sleepsec > 301: sleepsec = 302
             logging.warn('%s - reconnecting .. sleeping %s seconds' % (self.cfg.name, sleepsec))
-            time.sleep(sleepsec)
-
+            if not start: time.sleep(sleepsec)
+            #start = False
+            try:
+                if not start: self.exit(close=close)
+                if self.doreconnect(): break
+            except Exception, ex: logging.error(str(ex))
+            
     def doreconnect(self, start=False):
-        return self.start(connect=True)
+        self.started = False
+        return self.start()
 
-    def invite(self, *args, **kwargs):
-        """ invite another user/bot. """
-        pass
-
-    def donick(self, nick, *args, **kwargs):
-        """ do a nick change. """
-        pass
-
-    def shutdown(self, *args, **kwargs):
-        """ shutdown the bot. """
-        pass
-
-    def quit(self, reason="", *args, **kwargs):
-        """ close connection with the server. """
-        pass
-
-    def connect(self, reconnect=False, *args, **kwargs):
-        """ connect to the server. """
-        pass
-
-    def names(self, channel, *args, **kwargs):
-        """ request all names of a channel. """
-        pass
 
     def save(self, *args, **kwargs):
         """ save bot state if available. """
@@ -568,10 +538,6 @@ class BotBase(LazyDict):
             txt = u"<b>" + txt + u"</b>"
             for i in target:
                 if not i: continue
-                #if type(i) == types.ListType or type(i) == types.TupleType:
-                #    try:
-                #        res.append(dot.join(i))
-                #    except TypeError: res.extend(i)
                 if type(i) == types.DictType:
                     for key, value in i.iteritems():
                         res.append(u"%s: %s" % (key, unicode(value)))
@@ -612,32 +578,28 @@ class BotBase(LazyDict):
         e.botname = botname or self.cfg.name
         e.bottype = bottype or self.type
         e.origin = e.botname
-        e.ruserhost = self.cfg.name +'@' + self.cfg.uuid
-        e.userhost = e.ruserhost
+        e.userhost = self.cfg.name +'@' + self.cfg.uuid
+        e.nolog = True
         e.channel = botname
-        e.origtxt = "%s.%s - %s" % (e.botname, e.bottype, str(time.time()))
-        e.txt = e.origtxt
+        e.txt = "%s.%s - %s" % (e.botname, e.bottype, str(time.time()))
         e.cbtype = 'START'
         e.ttl = 1
         e.nick = self.cfg.nick or self.cfg.name
         self.doevent(e)
-        logging.info("%s - START event send to callbacks" % self.cfg.name)
+        logging.debug("%s - START event send to callbacks" % self.cfg.name)
 
     def outmonitor(self, origin, channel, txt, event=None):
         """ create an OUTPUT event with provided txt and send it to callbacks. """
         if event: e = cpy(event)
         else: e = EventBase()
-        #e = EventBase()
         if e.status == "done":
             logging.debug("%s - outmonitor - event is done .. ignoring" % self.cfg.name)
             return
         e.bot = self
         e.origin = origin
-        e.ruserhost = self.cfg.name +'@' + self.cfg.uuid
-        e.userhost = e.ruserhost
+        e.userhost = str(self.cfg.name) +'@' + str(self.cfg.uuid)
         e.auth = e.userhost
         e.channel = channel
-        e.origtxt = txt
         e.txt = txt
         e.cbtype = 'OUTPUT'
         e.nodispatch = True
@@ -645,40 +607,30 @@ class BotBase(LazyDict):
         e.nick = self.cfg.nick or self.cfg.name
         e.bonded = True
         e.isoutput = True
-        e.bind(self)
+        e.dontbind = True
         first_callbacks.check(self, e)
 
-    def make_event(self, origin, channel, txt, event=None, wait=0, showall=False, nooutput=False):
+    def make_event(self, origin, channel, txt, event=None, wait=0, showall=False, nooutput=False, cbtype=""):
         """ insert an event into the callbacks chain. """
-        assert origin
         if event: e = cpy(event)
-        else: e = EventBase()
-        e.cbtype = "CMND"
-        e.bot = self
-        e.origin = origin
-        e.ruserhost = origin
-        e.auth = origin
-        e.userhost = origin
+        else: e = EventBase(bot=self)
+        e.cbtype = cbtype or "CMND"
+        e.origin = origin or "test@test"
+        e.auth = e.origin
+        e.userhost = e.origin
         e.channel = channel
         e.txt = unicode(txt)
         e.nick = e.userhost.split('@')[0]
-        e.usercmnd = e.txt.split()[0]
         e.showall = showall
         e.nooutput = nooutput
         e.wait = wait
         e.closequeue = False
-        e.pipelined = False
+        e.bind(self)
         return e
 
     def execstr(self, origin, channel, txt, event=None, wait=0, showall=False, nooutput=False):
         e = self.make_event(origin, channel, txt, event, wait, showall, nooutput)
-        return e.eecwait()
-
-    def settopic(self, channel, txt):
-        pass
-
-    def gettopic(self, channel):
-        pass
+        return e.execwait()
 
     def docmnd(self, origin, channel, txt, event=None, wait=0, showall=False, nooutput=False):
         """ do a command. """
@@ -687,7 +639,6 @@ class BotBase(LazyDict):
         e.cbtype = "CMND"
         e.bot = self
         e.origin = origin
-        e.ruserhost = origin
         e.auth = origin
         e.userhost = origin
         e.channel = channel
@@ -700,3 +651,51 @@ class BotBase(LazyDict):
         e.nooutput = nooutput
         e.bind(self)
         if cmnds.woulddispatch(self, e) or e.txt[0] == "?": return self.doevent(e)
+
+    def join(self, channel, password, *args, **kwargs):
+        """ join a channel. """
+        pass
+
+    def part(self, channel, *args, **kwargs):
+        """ leave a channel. """
+        pass
+
+    def action(self, channel, txt, event=None, *args, **kwargs):
+        """ send action to channel. """
+        pass
+
+    def doop(self, channel, who):
+        """ give nick ops. """
+        pass
+
+    def invite(self, *args, **kwargs):
+        """ invite another user/bot. """
+        pass
+
+    def donick(self, nick, *args, **kwargs):
+        """ do a nick change. """
+        pass
+
+    def shutdown(self, *args, **kwargs):
+        """ shutdown the bot. """
+        pass
+
+    def quit(self, reason="", *args, **kwargs):
+        """ close connection with the server. """
+        pass
+
+    def connect(self, reconnect=False, *args, **kwargs):
+        """ connect to the server. """
+        pass
+
+    def names(self, channel, *args, **kwargs):
+        """ request all names of a channel. """
+        pass
+
+    def settopic(self, channel, txt):
+        pass
+
+    def gettopic(self, channel):
+        pass
+
+    def pingcheck(self): return True
