@@ -23,13 +23,13 @@ def get(name, namespace=""):
     """ get data from the cache. """
     global cache
     try: 
-        logging.debug("cache - returning %s" % cache[name])
+        #logging.debug("cache - returning %s" % cache[name])
         return cache[name]
     except KeyError: pass
 
 def set(name, item, timeout=0, namespace=""):
     """ set data in the cache. """
-    logging.debug("cache - setting %s to %s" % (name, str(item)))
+    #logging.debug("cache - setting %s to %s" % (name, str(item)))
     global cache
     cache[name] = item
 
