@@ -18,6 +18,21 @@ import sys
 class JsonBotError(Exception):
     pass
 
+class CantLogon(JsonBotError):
+    pass
+
+class URLNotEnabled(JsonBotError):
+    pass
+
+class JSONParseError(JsonBotError):
+    pass
+
+class StreamError(JsonBotError):
+    pass
+
+class RequireError(JsonBotError):
+    pass
+
 class CannotAuth(JsonBotError):
     pass
 
@@ -66,6 +81,9 @@ class NameNotSet(JsonBotError):
 class NoSuchUser(JsonBotError):
     pass
 
+class NoUserProvided(JsonBotError):
+    pass
+
 class NoSuchBotType(JsonBotError):
     pass
 
