@@ -28,7 +28,9 @@ import logging
 def globalshutdown(exit=True):
     """ shutdown the bot. """
     try:
-        logging.warn('shutting down'.upper())
+        try: sys.stdout.write("\n")
+        except: pass
+        logging.error('shutting down'.upper())
         from fleet import getfleet
         fleet = getfleet()
         if fleet:
@@ -47,9 +49,8 @@ def globalshutdown(exit=True):
         except: pass
         killmcdaemon()
         logging.warn('done')
-        time.sleep(0.1)
         if not ongae: print ""
         if exit and not ongae: os._exit(0)
     except Exception, ex:
-        handle_exception()
+        print str(ex)
         if exit and not ongae: os._exit(1)
