@@ -17,7 +17,7 @@ from jsb.utils.name import stripname
 from jsb.utils.locking import lockdec
 from jsb.utils.timeutils import elapsedstring
 from jsb.lib.callbacks import callbacks
-from jsb.lib.errors import MemcachedCounterError
+from jsb.lib.errors import MemcachedCounterError, JSONParseError
 
 from datadir import getdatadir
 
@@ -28,6 +28,7 @@ json = getjson()
 
 ## basic imports
 
+from collections import deque
 import thread
 import logging
 import os
@@ -41,15 +42,21 @@ import time
 
 cpy = copy.deepcopy
 
+## locks
+
+
+persistlock = thread.allocate_lock()
+persistlocked = lockdec(persistlock)
 
 ## global list to keeptrack of what persist objects need to be saved
 
-needsaving = []
+needsaving = deque()
 
 def cleanup(bot=None, event=None):
     global needsaving
+    todo = cpy(needsaving)
     r = []
-    for p in needsaving:
+    for p in todo:
         try: p.dosave() ; r.append(p) ; logging.warn("saved on retry - %s" % p.fn)
         except (OSError, IOError), ex: logging.error("failed to save %s - %s" % (p, str(ex)))
     for p in r:
@@ -57,13 +64,14 @@ def cleanup(bot=None, event=None):
         except ValueError: pass
     return needsaving
 
+
 ## try google first
 
 try:
-    import waveapi
+    from google.appengine.ext.db.metadata import Kind
     from google.appengine.ext import db
     import google.appengine.api.memcache as mc
-    from google.appengine.api.datastore_errors import Timeout
+    from google.appengine.api.datastore_errors import Timeout, TransactionFailedError
     from cache import get, set, delete
     logging.debug("using BigTable based Persist")
 
@@ -100,20 +108,21 @@ try:
             self.key = None
             self.obj = None
             self.size = 0
-            self.jsontxt = None
+            self.jsontxt = ""
             self.init(default)
 
         def init(self, default={}, filename=None):
-            cachetype = ""
             if self.checkmc(): self.jsontxt = self.updatemc() ; self.cachetype = "cache"
             else:
-                self.data = get(self.fn)
+                tmp = get(self.fn)
                 self.cachetype = "mem"
-                if self.data != None:
-                    logging.debug("source %s - loaded %s (%s)" % (self.cachetype, self.fn, str(self.data)))
+                if tmp != None:
+                    logging.warn("*%s* - loaded %s" % (self.cachetype, self.fn))
+                    self.data = tmp
                     if type(self.data) == types.DictType: self.data = LazyDict(self.data)
                     return self.data
-            if self.jsontxt is None:
+            if self.jsontxt == "": self.cachetype = "cache" ; self.jsontxt = mc.get(self.fn)
+            if self.jsontxt == None: 
                 self.cachetype = "db"
                 logging.debug("%s - loading from db" % self.fn) 
                 try:
@@ -126,7 +135,7 @@ try:
                         handle_exception()
                         self.obj = None
                 if self.obj == None:
-                    logging.debug("%s - no entry found" % self.fn)
+                    logging.warn("%s - no entry found, using default" % self.fn)
                     self.jsontext = json.dumps(default) ; self.cachetype = "default"
                 else:
                     self.jsontxt = self.obj.content; self.cachetype = "db"
@@ -137,18 +146,26 @@ try:
                         try: self.mcounter = self.counter = int(incr)
                         except ValueError: logging.error("can't make counter out of %s" % incr) 
                     else: self.mcounter = 1
-            logging.info("memcached counters for %s: %s" % (self.fn, self.mcounter))
+            logging.debug("memcached counters for %s: %s" % (self.fn, self.mcounter))
             if self.jsontxt == None: self.jsontxt = json.dumps(default) 
-            logging.info('%s - jsontxt is %s' % (self.fn, self.jsontxt))
-            self.data = json.loads(self.jsontxt)
+            logging.warn('%s - jsontxt is %s' % (self.fn, self.jsontxt))
+            try:
+                self.data = json.loads(self.jsontxt)
+            except: raise JSONParseError(self.fn)
             if not self.data: self.data = default
             self.size = len(self.jsontxt)
             if type(self.data) == types.DictType: self.data = LazyDict(self.data)
             set(self.fn, self.data)
-            logging.debug("source %s - loaded %s (%s)" % (self.cachetype, self.fn, len(self.jsontxt)))
+            logging.warn("*%s* - loaded %s (%s)" % (self.cachetype, self.fn, len(self.jsontxt)))
+
+        def get(self):
+            logging.debug("getting %s from local cache" % self.fn)
+            a = get(self.fn)
+            logging.debug("got %s from local cache" % type(a))
+            return a
 
         def sync(self):
-            logging.info("syncing %s" % self.fn)
+            logging.debug("syncing %s" % self.fn)
             tmp = cpy(self.data)
             data = json.dumps(tmp)
             mc.set(self.fn, data)
@@ -172,12 +189,23 @@ try:
             try:
                 self.mcounter = int(mc.get(self.countername)) or 0
             except: self.mcounter = 0
-            logging.debug("mcounter for %s is %s" % (self.fn, self.mcounter))
-            if (self.mcounter - self.counter) < 0: self.mcounter = self.counter = 0 ; return True
+            logging.warn("mcounter for %s is %s (%s)" % (self.fn, self.mcounter, self.counter))
+            if (self.mcounter - self.counter) < 0: return True
             elif (self.mcounter - self.counter) > 0: return True
             return False
-  
-        def save(self, filename=None):
+
+        def save(self):
+            cleanup()
+            global needsaving
+            try: self.dosave()
+            except (IOError, OSError, TransactionFailedError):
+                handle_exception()
+                logging.error("PUSHED ON RETRY QUEUE") 
+                self.sync()
+                if self not in needsaving: needsaving.appendleft(self)
+
+        @persistlocked
+        def dosave(self, filename=None):
             """ save json data to database. """
             if self.checkmc(): self.updatemc()
             fn = filename or self.fn
@@ -199,12 +227,41 @@ try:
                 except ValueError: logging.error("can't make counter out of %s" % incr) 
             else: self.mcounter = 1
             self.counter = self.mcounter
-            logging.info("memcached counters for %s: %s" % (fn, self.mcounter))
-            logging.warn('saved %s (%s) - %s' % (fn, len(bla), where()))
+            logging.debug("memcached counters for %s: %s" % (fn, self.mcounter))
+            logging.warn('saved %s (%s)' % (fn, len(bla)))
+            logging.debug('saved %s from %s' % (fn, where()))
 
         def upgrade(self, filename):
             self.init(self.data, filename=filename)
 
+    ## findfilenames function 
+
+    def findfilenames(target, filter=[], skip=[]):
+        res = []
+        targetkey = db.Key.from_path(JSONindb.kind(), target)
+        targetkey2 = db.Key.from_path(JSONindb.kind(), target + "zzz")
+        logging.warn("key for %s is %s" % (target, str(targetkey)))
+        q = db.Query(JSONindb, keys_only=True)
+        q.filter("__key__ >", targetkey)
+        q.filter("__key__ <", targetkey2)
+        for key in q:
+            fname = key.name()
+            if fname in skip: continue
+            fname = str(fname)
+            logging.warn("using %s" % fname)
+            go = True
+            for fil in filter:
+                if fil not in fname.lower(): go = False ; break
+            if not go: continue
+            res.append(fname)
+        return res
+
+    def findnames(target, filter=[], skip=[]):
+        res = []
+        for f in findfilenames(target, filter, skip):
+            res.append(f.split(os.sep)[-1])
+        return res
+
 
 except ImportError:
 
@@ -212,10 +269,6 @@ except ImportError:
 
     logging.debug("using file based Persist")
 
-    ## defines
-
-    persistlock = thread.allocate_lock()
-    persistlocked = lockdec(persistlock)
 
     ## imports for shell bots
 
@@ -229,7 +282,7 @@ except ImportError:
                 logging.warn("memcached uptime is %s" % elapsedstring(status[0][1]['uptime']))
                 got = True
         if got == False:
-            logging.info("no memcached found - using own cache")
+            logging.debug("no memcached found - using own cache")
         from cache import get, set, delete
 
     import fcntl
@@ -281,7 +334,7 @@ except ImportError:
             try:
                 logging.debug("using name %s" % self.fn)
                 a = get(self.fn)
-                if a: self.data = a.data
+                if a: self.data = a
                 else: self.data = None
                 if self.data != None:
                     logging.debug("got data from local cache")
@@ -312,8 +365,8 @@ except ImportError:
                     d = LazyDict()
                     d.update(self.data)
                     self.data = d
-                set(self.fn, self)
-                logging.info("loaded %s - %s" % (self.logname, cachetype))
+                set(self.fn, self.data)
+                logging.debug("loaded %s - %s" % (self.logname, cachetype))
             except Exception, ex:
                 logging.error('ERROR: %s' % self.fn)
                 raise
@@ -329,25 +382,25 @@ except ImportError:
             return a
 
         def sync(self):
-            logging.info("syncing %s" % self.fn)
+            logging.debug("syncing %s" % self.fn)
             if got: mc.set(self.fn, json.dumps(self.data))
-            set(self.fn, self)
+            set(self.fn, self.data)
             return self
 
         def save(self):
+            cleanup()
             global needsaving
             try: self.dosave()
             except (IOError, OSError):
                 self.sync()
                 if self not in needsaving: needsaving.append(self)
-        
+
         @persistlocked
         def dosave(self):
             """ persist data attribute. """
             try:
                 if self.dontsave: logging.error("dontsave is set on  %s - not saving" % self.fn) ; return
                 fn = self.fn
-                #print help(mc.incr)
                 if got: self.mcounter = int(mc.incr(self.countername))
                 if got and (self.mcounter - self.counter) > 1:
                     tmp = json.loads(mc.get(fn))
@@ -362,7 +415,7 @@ except ImportError:
                     d.append(p)
                     pp = os.sep.join(d)
                     if not os.path.isdir(pp):
-                        logging.info("creating %s dir" % pp)
+                        logging.warn("creating %s dir" % pp)
                         os.mkdir(pp)
                 tmp = fn + '.tmp' # tmp file to save to
                 datafile = open(tmp, 'w')
@@ -377,13 +430,36 @@ except ImportError:
                 jsontxt = json.dumps(self.data)
                 logging.debug("setting cache %s - %s" % (fn, jsontxt))
                 self.jsontxt = jsontxt
-                set(fn, self)
+                set(fn, self.data)
                 if got: mc.set(fn, jsontxt)
-                logging.warn('%s saved' % self.logname)
+                logging.info('%s saved' % self.logname)
             except IOError, ex: logging.error("not saving %s: %s" % (self.fn, str(ex))) ; raise
             except: raise
             finally: pass
 
+    ## findfilenames function 
+
+    def findfilenames(target, filter=[], skip=[]):
+        res = []
+        if not os.path.isdir(target): return res
+        for f in os.listdir(target):
+            if f in skip: continue
+            fname = target + os.sep + f
+            if os.path.isdir(fname): res.extend(findfilenames(fname, skip))
+            go = True
+            for fil in filter:
+                if fil not in fname.lower(): go = False ; break
+            if not go: continue
+            res.append(fname)
+        return res
+
+    def findnames(target, filter=[], skip=[]):
+        res = []
+        for f in findfilenames(target, filter, skip):
+            res.append(f.split(os.sep)[-1])
+        return res
+
+
 class PlugPersist(Persist):
 
     """ persist plug related data. data is stored in jsondata/plugs/{plugname}/{filename}. """
@@ -401,4 +477,71 @@ class GlobalPersist(Persist):
         logging.warn("filename is %s" % filename)
         Persist.__init__(self, getdatadir() + os.sep + 'globals' + os.sep + stripname(filename), default=default, *args, **kwargs)
 
+## PersistCollection class
+
+class PersistCollection(object):
+
+    """ maintain a collection of Persist objects. """
+
+    def __init__(self, path, *args, **kwargs):
+        assert path
+        self.path = path
+        d = [os.sep, ]
+        for p in path.split(os.sep):
+            if not p: continue
+            d.append(p)
+            pp = os.sep.join(d)
+            try:
+                os.mkdir(pp)
+                logging.warn("creating %s dir" % pp)
+            except OSError, ex:
+                if 'Errno 13' in str(ex) or 'Errno 2' in str(ex): continue
+                logging.warn("can't make %s - %s" % (pp,str(ex))) ; continue
+                
+    def filenames(self, filter=[], path=None, skip=[], result=[]):
+        target = path or self.path
+        res = findfilenames(target, filter, skip)
+        logging.info("filenames are %s" % str(res))
+        return res
+
+    def names(self, filter=[], path=None, skip=[], result=[]):
+        target = path or self.path
+        res = findnames(target, filter, skip)
+        return res
+
+    def search(self, field, target):
+        res = []
+        for obj in self.objects().values():
+            try: item = getattr(obj.data, field)
+            except AttributeError: handle_exception() ; continue
+            if not item: continue
+            if target in item: res.append(obj)
+        return res
+            
+    def objects(self, filter=[], path=None):
+        if type(filter) != types.ListType: filter = [filter, ] 
+        res = {}
+        target = path or self.path
+        for f in self.filenames(filter, target):
+             res[f] = Persist(f)
+        return res
+
+## PlugPersistCollection class
+
+class PlugPersistCollection(PersistCollection):
+
+    def __init__(self):
+        plugname = calledfrom(sys._getframe())
+        self.path =  getdatadir() + os.sep + 'plugs' + os.sep + stripname(plugname) + os.sep
+        PersistCollection.__init__(self, self.path)
+
+## GlobalPersistCollection class
+
+class GlobalPersistCollection(PersistCollection):
+
+    def __init__(self):
+        self.path =  getdatadir() + os.sep + 'globals'
+        GlobalCollection(self, self.path)
+
+
 callbacks.add("TICK60", cleanup)
