@@ -32,7 +32,8 @@ def reboot_stateful(bot, ievent, fleet, partyline):
     """ reboot the bot, but keep the connections (IRC only). """
     logging.warn("reboot - doing statefull reboot")
     session = {'bots': {}, 'name': bot.cfg.name, 'channel': ievent.channel, 'partyline': []}
-    for i in getfleet().bots:
+    fleet = getfleet()
+    for i in fleet.bots:
         logging.warn("reboot - updating %s" % i.cfg.name)
         data = i._resumedata()
         if not data: continue
@@ -45,7 +46,13 @@ def reboot_stateful(bot, ievent, fleet, partyline):
             for socketlist in i.websockets.values():
                 for sock in socketlist: sock.stream.close()
     session['partyline'] = partyline._resumedata()
-    sessionfile = tempfile.mkstemp('-session', 'jsb-')[1]
-    json.dump(session, open(sessionfile, 'w'))
-    getfleet().save()
-    os.execl(sys.argv[0], sys.argv[0], '-r', sessionfile, *sys.argv[1:])
+    sfile, sessionfile = tempfile.mkstemp('-session', 'jsb-', text=True)
+    logging.warn("writing session file %s" % sessionfile)
+    json.dump(session, open(sessionfile, "w"))
+    args = []
+    skip = False
+    for a in sys.argv[1:]:
+        if skip: skip = False ; continue
+        if a == "-r": skip = True ; continue
+        args.append(a)
+    os.execl(sys.argv[0], sys.argv[0], '-r', sessionfile, *args)
