@@ -9,6 +9,7 @@
 from jsb.lib.threadloop import TimedLoop
 from jsb.lib.eventbase import EventBase
 from jsb.lib.callbacks import callbacks
+from jsb.lib.config import getmainconfig
 
 ## TickLoop class
 
@@ -25,11 +26,17 @@ class TickLoop(TimedLoop):
         """ send TICK events to callback. """
         self.counter += 1
         event = EventBase()
-        event.type = event.cbtype = 'TICK60'
-        callbacks.check(self.bot, event)
-        #event.type = event.cbtype = 'TICK'
-        #callbacks.check(self.bot, event)
+        event.nolog = True
+        event.nobind = True
+        if self.counter % 60 == 0:
+            event.type = event.cbtype = 'TICK60'
+            callbacks.check(self.bot, event)
+        maincfg = getmainconfig()
+        t = maincfg.ticksleep or 1
+        if self.counter % t == 0:
+            event.type = event.cbtype = 'TICK'
+            callbacks.check(self.bot, event)
 
 ## global tick loop
 
-tickloop = TickLoop('tickloop', 60)
+tickloop = TickLoop('tickloop', 1)
