@@ -53,7 +53,7 @@ class BugTrackerNotSupported(Exception):
 class BugTracker:
 
     bugurls = [] # regexp patterns that could match a bugtracker url
-    bugtags = re.compile('(?:bug|ticket)\s?#(\d+)', re.I)
+    bugtags = re.compile('(?:bug|ticket)*\s?#(\d+)', re.I)
 
     def __init__(self, baseUrl, autoshow):
         self.url = baseUrl.rstrip('/')
@@ -88,6 +88,8 @@ class BugTracker:
 
     def show_url(self, bugId):
         raise BugTrackerNotImplemented
+
+    ## bug-close command
         
     def handle_close(self, bot, ievent):
         if len(ievent.args) < 3:
@@ -106,6 +108,8 @@ class BugTracker:
         except BugTrackerNotFound, e:
             ievent.reply('error: not found: %s' % e)
 
+    ## bug-comments command
+
     def handle_comments(self, bot, ievent):
         if not ievent.args or not ievent.args[0].isdigit():
             ievent.missing('<bug id>')
@@ -125,6 +129,8 @@ class BugTracker:
         except BugTrackerNotFound, e:
             ievent.reply('error: not found: %s' % e)
 
+    ## bug-list command
+
     def handle_list(self, bot, ievent):
         try:
             bugs = self.list()
@@ -139,6 +145,8 @@ class BugTracker:
         except BugTrackerNotFound, e:
             ievent.reply('error: not found: %s' % e)
 
+    ## bug-show command
+
     def handle_show(self, bot, ievent):
         if len(ievent.args) != 1:
             try:
@@ -156,14 +164,11 @@ class BugTracker:
                     result.append('%s=%s' % (key, status[key]))
             #result.append(self.show_url(ievent.args[0]))
             ievent.reply("bug #%s: " % ievent.args[0], result)
-        except AssertionError, e:
-            ievent.reply('error: %s' % e)
-        except BugTrackerNotImplemented:
-            ievent.reply('error: not implemented in this bug tracker')
-        except BugTrackerNotSupported, e:
-            ievent.reply('error: not supported: %s' % e)
-        except BugTrackerNotFound, e:
-            ievent.reply('error: not found: %s' % e)
+        except urllib2.HTTPError, e: ievent.reply("no #%s bug found: %s" % (ievent.args[0], str(e))) ; return
+        except AssertionError, e: ievent.reply('error: %s' % e)
+        except BugTrackerNotImplemented: ievent.reply('error: not implemented in this bug tracker')
+        except BugTrackerNotSupported, e: ievent.reply('error: not supported: %s' % e)
+        except BugTrackerNotFound, e: ievent.reply('error: not found: %s' % e)
 
     def scan_privmsg(self, bot, ievent):
         if not self.autoshow:
@@ -173,14 +178,14 @@ class BugTracker:
         for bugurl in self.bugurls:
             test = bugurl.findall(ievent.origtxt)
             if test:
-                nevent = copy.copy(ievent)
+                nevent = copy.deepcopy(ievent)
                 nevent.args = [test[0]]
                 self.handle_show(bot, nevent)
                 del nevent
                 return
         test = self.bugtags.findall(ievent.origtxt)
         if test:
-            nevent = copy.copy(ievent)
+            nevent = copy.deepcopy(ievent)
             nevent.args = [test[0]]
             self.handle_show(bot, nevent)
             del nevent
@@ -459,15 +464,18 @@ class Trac(BugTracker):
 
     def list(self):
         tsv = geturl2(self.list_url()).splitlines()
+        #print tsv
         bugs = []
 	keys = map(lambda x: x.strip(), tsv[0].split())
 	for item in tsv[1:]:
 		data = {}
 		part = item.split('\t')
 		# dirty hack to allow unescaped multilines in trac
-		if part[0].isdigit() and part[1].isdigit() and len(part) == len(keys):
-			data = dict(zip(keys, part))
-			bugs.append('%s (%s)' % (data['summary'], data['ticket']))
+		#if part[0].isdigit() and part[1].isdigit() and len(part) == len(keys):
+                if True:
+                    #data = dict(zip(keys, part))
+                    try: bugs.append('%s (%s)' % (part[1], part[0]))
+                    except IndexError: pass
         return bugs
 
     def list_url(self):
@@ -669,6 +677,7 @@ class BugManager(Pdod):
             return False
         try:
             self.instances[bot.cfg.name][ievent.channel].handle_show(bot, ievent)
+        except KeyError, ex: ievent.reply("key error: %s" % str(ex)) ; return False
         except Exception, e:
             handle_exception()
             ievent.reply('failed to fetch bug information: %s' % str(e))
@@ -680,20 +689,24 @@ class BugManager(Pdod):
             return 0
         self.instances[bot.cfg.name][ievent.channel].scan_privmsg(bot, ievent)
 
+## define
+
 bugtrackers = BugManager()
 
+## plugin init
+
 def init():
     callbacks.add('PRIVMSG', bugtrackers.cb_privmsg)
 
 cmnds.add('bug-autoshow', bugtrackers.handle_autoshow, ['OPER'])
 examples.add('bug-autoshow', 'toggle automatic retrieval of bug information', 'bug-autoshow')
-cmnds.add('bug-close', bugtrackers.handle_close, ['OPER', 'BUGS'])
+cmnds.add('bug-close', bugtrackers.handle_close, ['OPER', 'BUGS'], threaded=True)
 examples.add('bug-close', 'close a bug', 'bug-close 42 fixed Typo on line 23')
-cmnds.add('bug-comments', bugtrackers.handle_comments, ['USER'])
+cmnds.add('bug-comments', bugtrackers.handle_comments, ['USER'], threaded=True)
 examples.add('bug-comments', 'show all comments on a bug', 'bug-comments 42')
-cmnds.add('bug-list', bugtrackers.handle_list, ['USER', 'BUGS'])
+cmnds.add('bug-list', bugtrackers.handle_list, ['USER', 'BUGS'], threaded=True)
 examples.add('bug-list', 'list all open bugs', 'bug-list')
-cmnds.add('bug', bugtrackers.handle_show, ['USER'])
+cmnds.add('bug', bugtrackers.handle_show, ['USER'], threaded=True)
 examples.add('bug', 'show a bug', 'bug 42')
 cmnds.add('bug-info', bugtrackers.handle_tracker_info, ['BUGSOPER', 'OPER'])
 cmnds.add('bug-set', bugtrackers.handle_tracker_set, ['BUGSOPER', 'OPER'])
