@@ -679,7 +679,7 @@ examples.add('hb-addchannel', 'add a channel to a feeds watchlist', '1) hb-addch
 def handle_hubbubsetitems(bot, ievent):
     """ arguments: <feedname> <list of tokens> - set items (tokens) of a feed, see hb-scan for a list of possible tokens. """
     try: (name, items) = ievent.args[0], ievent.args[1:]
-    except ValueError: ievent.missing('<feedname> <tokens>') ; return
+    except (ValueError, IndexError): ievent.missing('<feedname> <tokens>') ; return
     target = ievent.channel
     feed =  watcher.byname(name)
     if not feed: ievent.reply("we don't have a %s feed" % name) ; return
