@@ -20,7 +20,7 @@
 from jsb.lib.persist import Persist, PlugPersist
 from jsb.utils.url import geturl2, striphtml, useragent
 from jsb.utils.exception import handle_exception
-from jsb.utils.generic import strippedtxt, fromenc, toenc, jsonstring, getwho
+from jsb.utils.generic import strippedtxt, fromenc, toenc, jsonstring, getwho, getnick
 from jsb.utils.rsslist import rsslist
 from jsb.utils.lazydict import LazyDict
 from jsb.utils.statdict import StatDict
@@ -212,7 +212,9 @@ sleeptime=15*60, running=0):
     def fetchdata(self, data=None):
         """ get data of rss feed. """
         name = self.data.name
-        #logging.warn("fetching %s" % name)
+        from jsb.utils.url import enabled
+        if not enabled: return []
+        logging.info("fetching %s" % name)
         global etags
         if name and etags.data.has_key(name): etag = etags.data[name]
         else: etag = None
@@ -222,11 +224,11 @@ sleeptime=15*60, running=0):
             except AttributeError: status = None
         else:
             url = self.data['url']
-            logging.warn("fetching %s" % strippassword(url))
+            logging.info("fetching %s" % strippassword(url))
             result = feedparser.parse(url, agent=useragent(), etag=etag)
             try: status = result.status
             except AttributeError: status = None
-        logging.warn("status returned of %s feed is %s" % (name, status))
+        logging.info("status returned of %s feed is %s" % (name, status))
         if status == 304: return []
         if result: set(self.data.url, result.entries, namespace='rss')
         if data:
@@ -249,11 +251,7 @@ sleeptime=15*60, running=0):
             return False
         logging.info("syncing %s - %s" % (self.data.name, strippassword(self.data.url)))
         result = self.fetchdata()
-        #if not result:
-        #    cached = get(self.data.url, namespace="rss")
-        #    if cached: result = cached
-        #    else: result = []
-        logging.warn("%s entries fetched" % len(result))
+        logging.info("%s entries fetched" % len(result))
         return result or []
 
     def updateseen(self, entries=None):
@@ -263,7 +261,7 @@ sleeptime=15*60, running=0):
         if entries:
             for res in entries[::-1]: self.setseen(res)
         self.save(coreonly=True)
-        logging.warn("%s - %s items updated" % (self.data.name, len(entries)))
+        logging.info("%s - %s items updated" % (self.data.name, len(entries)))
 
     def check(self, entries=None, save=True):
         got = False
@@ -296,24 +294,26 @@ sleeptime=15*60, running=0):
                 if not type: logging.error("%s - %s is not correct" % (name, str(item))) ; continue
                 try:
                     bot = getfleet().byname(botname)
-                    if not bot: bot = getfleet().makebot(type, botname)
+                    if not bot and ongae: bot = getfleet().makebot(type, botname)
                 except NoSuchBotType, ex: logging.warn("can't make bot - %s" % str(ex)) ; continue
-                if not bot: logging.error("can't find %s bot in fleet" % botname) ; continue
-                if bot.type == "irc": bot.connectok.wait()
+                if not bot: logging.warn("can't find %s bot in fleet" % botname) ; continue
+                if bot.type == "irc": logging.debug("waiting for irc bot to connect") ; bot.connectok.wait()
                 res2 = datalist
-                if type == "irc" and not '#' in channel: nick = getwho(bot, channel)
-                else: nick = None                        
+                if type == "irc" and not '#' in channel:
+                     nick = getnick(bot, channel)
+                     if not nick: logging.error("can't get nick to send to %s" % channel) ; continue
+                else: nick = None
                 if self.markup.get(jsonstring([name, type, channel]), 'reverse-order'): res2 = res2[::-1]
                 if self.markup.get(jsonstring([name, type, channel]), 'all-lines'):
                     for i in res2: 
                         response = self.makeresponse(name, type, [i, ], channel)
-                        try: bot.saynocb(nick or channel, response, **{'headlines': True})
+                        try: bot.say(nick or channel, response, **{'headlines': True})
                         except Exception, ex: handle_exception()
                 else:
                     sep =  self.markup.get(jsonstring([name, type, channel]), 'separator')
                     if sep: response = self.makeresponse(name, type, res2, channel, sep=sep)
                     else: response = self.makeresponse(name, type, res2, channel)
-                    try: bot.saynocb(nick or channel, response, **{'headlines': True})
+                    try: bot.say(nick or channel, response, **{'headlines': True})
                     except Exception, ex: handle_exception()
             return True
         except Exception, ex: handle_exception(txt=name) ; return False
@@ -675,7 +675,7 @@ class Rsswatcher(Rssdict):
         rssitem.data.running = 1
         rssitem.data.stoprunning = 0
         rssitem.save()
-        if name not in self.runners(): watcher.watch(name)
+        #if name not in self.runners(): watcher.watch(name)
         logging.warn("started %s feed in %s channel" % (name, channel))
         return True
 
@@ -778,15 +778,15 @@ def doperiodicalGAE(*args, **kwargs):
     curtime = time.time()
     feedstofetch = []
     rpcs = []
-    for feed in watcher.runners():
-        if not watcher.shouldpoll(feed, curtime): continue
+    for feed in watcher.data.names:
+        if not shouldpoll(feed, curtime): continue
         feedstofetch.append(feed)
-    logging.info("feeds to fetch: %s" % str(feedstofetch))
+    logging.warn("feeds to fetch: %s" % str(feedstofetch))
     got = False
     for f in feedstofetch:
         if not f: continue
         lastpoll.data[f] = curtime
-        if f not in urls.data: url = Feed(f).data.url ; urls.data[f] = url ; got = True
+        if f not in urls.data: logging.warn("adding %s to urls" % f) ; url = Feed(f).data.url ; urls.data[f] = url ; got = True
         else: url = urls.data[f]
         rpc = urlfetch.create_rpc()
         rpc.feedname = f
@@ -860,8 +860,8 @@ def handle_rssclone(bot, event):
     feeds = watcher.clone(bot.cfg.name, bot.type, event.channel, event.rest)
     event.reply('cloned the following feeds: ', feeds)
  
-cmnds.add('rss-clone', handle_rssclone, 'USER')
-examples.add('rss-clone', 'clone feeds into new channel', 'rss-clone #dunkbots')
+#cmnds.add('rss-clone', handle_rssclone, 'USER')
+#examples.add('rss-clone', 'clone feeds into new channel', 'rss-clone #dunkbots')
 
 ## rss-cloneurl command
 
@@ -871,8 +871,8 @@ def handle_rsscloneurl(bot, event):
     feeds = watcher.cloneurl(event.rest, event.auth)
     event.reply('cloned the following feeds: ', feeds)
  
-cmnds.add('rss-cloneurl', handle_rsscloneurl, 'OPER')
-examples.add('rss-cloneurl', 'clone feeds from remote url', 'rss-cloneurl http://jsonbot.org/feeds.opml')
+#cmnds.add('rss-cloneurl', handle_rsscloneurl, 'OPER')
+#examples.add('rss-cloneurl', 'clone feeds from remote url', 'rss-cloneurl http://jsonbot.org/feeds.opml')
 
 ## rss-add command
 
@@ -881,9 +881,9 @@ def handle_rssadd(bot, ievent):
     try: (name, url) = ievent.args
     except ValueError: ievent.missing('<feedname> <url>') ; return
     if watcher.checkfeed(url, ievent):
-         feed = watcher.add(name, url, ievent.userhost)
-         feed.updateseen()
-         ievent.reply('rss item added')
+        feed = watcher.add(name, url, ievent.userhost)
+        feed.updateseen()
+        ievent.reply('rss item added')
     else: ievent.reply('%s is not valid' % strippassword(url))
 
 cmnds.add('rss-add', handle_rssadd, 'USER')
@@ -900,7 +900,7 @@ def handle_rssregister(bot, ievent):
         return
     if watcher.checkfeed(url, ievent):
         feed = watcher.add(name, url, ievent.userhost)
-        #watcher.watch(name)
+        feed.updateseen()
         watcher.start(bot.cfg.name, bot.type, name, ievent.channel)
         if name not in ievent.chan.data.feeds: ievent.chan.data.feeds.append(name) ; ievent.chan.save()
         ievent.reply('rss item added and started in channel %s' % ievent.channel)
@@ -1260,7 +1260,7 @@ def handle_rsssetsleeptime(bot, ievent):
         except KeyError, ex: ievent.reply("failed to set interval: %s" % str(ex)) ; return
     ievent.reply('sleeptime set')
 
-cmnds.add('rss-setsleeptime', handle_rsssetsleeptime, ['USER', ])
+cmnds.add('rss-setsleeptime', handle_rsssetsleeptime, ['OPER', ])
 examples.add('rss-setsleeptime', 'set sleeping time of a feed .. min 60 sec', 'rss-setsleeptime jsonbot 600')
 
 ## rss-get command
@@ -1293,7 +1293,7 @@ def handle_rssrunning(bot, ievent):
     if resultlist: ievent.reply("running rss watchers: ", resultlist)
     else: ievent.reply('nothing running yet')
 
-cmnds.add('rss-running', handle_rssrunning, ['RSS', 'USER'])
+cmnds.add('rss-running', handle_rssrunning, ['RSS', 'OPER'])
 examples.add('rss-running', 'rss-running .. get running rsswatchers', \
 'rss-running')
 
@@ -1336,7 +1336,7 @@ def handle_rssseturl(bot, ievent):
         ievent.reply('url of %s changed' % name)
     else: ievent.reply('failed to set url of %s to %s' % (name, strippassword(url)))
 
-cmnds.add('rss-seturl', handle_rssseturl, ['USER', ])
+cmnds.add('rss-seturl', handle_rssseturl, ['OPER', ])
 examples.add('rss-seturl', 'set the url of a feed', 'rss-seturl jsonbot-hg http://code.google.com/feeds/p/jsonbot/hgchanges/basic')
 
 ## rss-itemslist
