@@ -20,10 +20,10 @@ import time
 def handle_choice(bot, ievent):
     """ arguments: [<space seperated strings>] - make a random choice out of different words or list elements. when used in a pipeline will choose from that. """ 
     result = []
-    if ievent.args: result = ievent.args
-    else:
-         if not ievent.inqueue: time.sleep(0.5)
+    if ievent.prev:
+         ievent.prev.wait()
          result = ievent.inqueue
+    else: result = ievent.args
     if result: ievent.reply(random.choice(result))
     else: ievent.reply("no result")
 
