@@ -18,8 +18,10 @@ import time
 
 def handle_count(bot, ievent):
     """ no arguments - show nr of elements in result list .. use this command in a pipeline. """
-    if not ievent.inqueue: time.sleep(0.5)
-    ievent.reply(str(len(ievent.inqueue)))
+    #if ievent.prev: ievent.prev.wait()
+    a = ievent.inqueue
+    size = len(a)
+    ievent.reply(size)
 
 cmnds.add('count', handle_count, ['OPER', 'USER', 'GUEST'])
 examples.add('count', 'count nr of items', 'list ! count')
