@@ -6,6 +6,8 @@
 
 ## jsb imports
 
+from jsb.lib.aliases import setalias
+from jsb.lib.config import getmainconfig
 from jsb.lib.commands import cmnds
 from jsb.lib.examples import examples
 
@@ -13,7 +15,7 @@ from jsb.lib.examples import examples
 
 def handle_dataevent(bot, event):
     """ no arguments - dump event to json. """
-    event.reply(event.tojson())
+    event.reply(event.fordisplay())
 
 cmnds.add("data-event", handle_dataevent, "OPER")
 examples.add('data-event', 'dump event data', 'data-event')
@@ -22,7 +24,7 @@ examples.add('data-event', 'dump event data', 'data-event')
 
 def handle_datachan(bot, event):
     """ no arguments - dump channel data to json. """
-    event.reply(event.chan.data.tojson())
+    event.reply(event.chan.data.fordisplay())
 
 cmnds.add("data-chan", handle_datachan, "OPER")
 examples.add('data-chan', 'dump channel data', 'data-chan')
@@ -31,7 +33,24 @@ examples.add('data-chan', 'dump channel data', 'data-chan')
 
 def handle_databot(bot, event):
     """ no arguments - dump bot as json dict. """
-    event.reply(bot.tojson())
+    event.reply(bot.fordisplay())
 
 cmnds.add("data-bot", handle_databot, "OPER")
 examples.add('data-bot', 'dump bot data', 'data-bot')
+
+## data-cfg command
+
+def handle_datacfg(bot, event):
+    """ no arguments - dump bot as json dict. """
+    event.reply(bot.cfg.fordisplay())
+
+cmnds.add("data-cfg", handle_datacfg, "OPER")
+examples.add('data-cfg', 'dump bot config data', 'data-cfg')
+
+def handle_datamain(bot, event):
+    """ no arguments - dump bot as json dict. """
+    event.reply(getmainconfig().fordisplay())
+
+cmnds.add("data-main", handle_datamain, "OPER")
+examples.add('data-main', 'dump mainconfig data', 'data-main')
+
