@@ -23,8 +23,10 @@ import time
 def handle_reverse(bot, ievent):
     """ arguments: [<string>] - reverse string or use in a pipeline. """
     if ievent.rest: ievent.reply(ievent.rest[::-1]) ; return
-    if not ievent.inqueue: time.sleep(0.5)
-    ievent.reply("results: ", list(ievent.inqueue)[::-1])
+    if True or ievent.prev:
+        #ievent.prev.wait()
+        ievent.reply("results: ", reversed(ievent.inqueue))
+        return
 
-cmnds.add('reverse', handle_reverse, ['OPER', 'USER', 'GUEST'])
+cmnds.add('reverse', handle_reverse, ['USER', 'GUEST'])
 examples.add('reverse', 'reverse text or pipeline', '1) reverse gozerbot 2) list ! reverse')
