@@ -18,12 +18,13 @@ import sys
 
 def handle_size(bot, event):
     res = []
-    for name, mod in sys.modules.iteritems():
+    mods = dict(sys.modules)
+    for name, mod in mods.iteritems():
        if not 'jsb' in name: continue
        try: res.append("<i><%s></i> %s" % (name.split(".")[-1], unicode(getattr(mod, 'size')())))
        except (TypeError, AttributeError): continue
        except Exception, ex: handle_exception()
-    event.reply("sizes in %s modules scanned: " % len(res), res, dot=", ", html=True)
+    event.reply("sizes in %s modules scanned: " % len(res), res, dot=", ")
 
 cmnds.add("size", handle_size, "OPER")
 examples.add("size", "call size() functions in all available modules", "size")
