@@ -6,7 +6,7 @@
 
 ## jsb imports
 
-from jsb.lib.datadir import getdatadir
+from jsb.lib.config import getmainconfig
 
 ## basic imports
 
@@ -15,10 +15,11 @@ import binascii
 
 ## defines
 
-version = "0.80.3"
+version = "0.84.4"
+__version__ = version
 
 ## getversion function
 
 def getversion(txt=""):
     """ return a version string. """
-    return "JSONBOT %s RELEASE %s" % (version, txt or "!")
+    return "JSONBOT %s RELEASE %s" % (version, txt)
